import datetime
from pathlib import Path
from typing import Union

import numpy as np
import pandas as pd
from radiens.grpc_radiens import biointerface_pb2, common_pb2
from radiens.utils.constants import (BYTES_PER_SAMPLE, TIME_SPEC, KeyIdxs,
                                     NeighborsDesc, NeuronDesc,
                                     NeuronExtendedMetadata, SigSelect,
                                     SitePosition, TimeRange)
from radiens.utils.enums import KeyIndex


def time_now():
    return datetime.datetime.now().strftime(TIME_SPEC)


def file_types(category='all'):
    '''
    File extensions for supported Radiens file types. 

    Parameters:
        category (str): optional, requested file category: 'all', 'recording', 'spikes', default='all'

    Returns:
        ext (list): list of requested category file extensions.

    Example:
        >>> file_types("recording")
        ['rhd', 'xdat', 'csv', 'hdf5', 'h5', 'nwb', 'kilosort2', 'nsx', 'pl2', 'tdt']

    '''
    rec = ['rhd', 'xdat', 'csv', 'hdf5', 'h5',
           'nwb', 'kilosort2', 'nsx', 'pl2', 'tdt']
    spikes = ['spikes']
    if category in ['recording']:
        return rec
    if category in ['spikes']:
        return spikes
    if category in ['all']:
        rec.extend(spikes)
        return rec
    raise ValueError("category must be 'rec', 'spikes', 'all'")


def is_xdat_file(p: Path) -> bool:
    return Path(p.parent, p.stem+'_data.xdat').expanduser().resolve().is_file()


def rm_xdat_file(p: Path) -> None:
    if not is_xdat_file(p):
        return
    Path(p.parent, p.stem+'_data.xdat').expanduser().resolve().unlink()
    Path(p.parent, p.stem+'_timestamp.xdat').expanduser().resolve().unlink()
    Path(p.parent, p.stem+'.xdat.json').expanduser().resolve().unlink()


def make_site_pos(probe, tissue):
    return SitePosition(probe_x=probe[0], probe_y=probe[1], probe_z=probe[2],
                        tissue_x=tissue[0], tissue_y=tissue[1], tissue_z=tissue[2])


def make_time_range(time_range: Union[list, np.ndarray] = None, timestamp: Union[list, np.ndarray] = None, fs: float = None, walltime: datetime.datetime = None, pbTR: common_pb2.TimeRange = None):
    '''
    Utility function to compose the TimeRange named tuple composed from either a time range or timestamp list. 

    Parameters:
        time_range (list | numpy.ndarray): time range in seconds as [time start, time end] (optional, default=None). 
        timestamp (list | numpy.ndarray): timestamp in seconds as [timestamp start, timestamp end]  (optional, default=None). 
        fs (float): sampling frequency in samples/sec (required, default=None)
        walltime (datetime.datetime): wall time of the time range start.
        pbTR : low-level system parameter not available to user.

    Notes:

    TimeRange named tuple fields: 
        `sec` (numpy.ndarray): [time start, time end]
        `timestamp` (numpy.ndarray): [timestamp start, timestamp end]
        `fs` (float): sampling frequency
        `walltime`: wall time of the time range start
        `dur_sec` (float): imputed time range duration equal to `TR.time_range[1]`-`TR.time_range[0]`.
        `N` (int): imputednumber of sample points equal to `TR.timestamp[1]`-`TR.timestamp[0]`.

    Either `timestamp` or `time_range` must be provided, with `timestamp` having preference. 
    `walltime` is optional.  `dur_sec` and `N` are imputed from the arguments. 

    System use only (not available to user)
    Either `pbTR` or (`fs` and either `time_range` or `timestamp`) must be provided as arguments. 
    `pbTR` has preference.  
    '''
    if isinstance(pbTR, common_pb2.TimeRange):
        return time_range_from_protobuf(pbTR)
    if fs in [None]:
        raise ValueError('missing fs')
    if walltime in [None]:
        walltime = datetime.datetime.min
    if isinstance(timestamp, (list, np.ndarray)):
        ts = np.array(timestamp).astype(np.int64)
        tr = np.array([ts[0]/fs, ts[1]/fs], dtype=np.float64)
    elif isinstance(time_range, (list, np.ndarray)):
        tr = np.array(time_range, dtype=np.float64)
        ts = np.array([tr[0]*fs, tr[1]*fs]).astype(np.int64)
    else:
        raise ValueError('missing both time_range and timestamp arg')
    return TimeRange(sec=tr, timestamp=ts, fs=fs, walltime=walltime, N=ts[1]-ts[0], dur_sec=tr[1]-tr[0])


def time_range_to_protobuf(tr: TimeRange) -> common_pb2.TimeRange:
    if not isinstance(tr, TimeRange):
        raise ValueError('argument must be a TimeRange')
    if len(tr.timestamp) != 2:
        raise ValueError('argument timestamp field does not have len=2')
    pbTR = common_pb2.TimeRange(timestamp=tr.timestamp.tolist(), fs=tr.fs)
    pbTimestamp = common_pb2.google_dot_protobuf_dot_timestamp__pb2.Timestamp()
    pbTimestamp.FromDatetime(tr.walltime)
    pbTR.wallTime.CopyFrom(pbTimestamp)
    return pbTR


def is_protobuf_time_range_empty(pbTR: common_pb2.TimeRange) -> bool:
    if pbTR.fs == 0 or (len(pbTR.timestamp) == 0 and len(pbTR.timeRangeSec) == 0):
        return True
    return False


def time_range_from_protobuf(pbTR: common_pb2.TimeRange) -> TimeRange:
    if not isinstance(pbTR, common_pb2.TimeRange):
        raise ValueError('argument must be a common_pb2.TimeRange')
    if pbTR.timestamp not in [None] and len(pbTR.timestamp) == 2:
        ts = np.array(pbTR.timestamp).astype(np.int64)
        tr = np.array([ts[0]/pbTR.fs, ts[1]/pbTR.fs], dtype=np.float64)
    elif pbTR.timeRangeSec not in [None] and len(pbTR.timeRangeSec) == 2:
        tr = np.array(pbTR.timeRangeSec).astype(np.float64)
        ts = np.array([tr[0]*pbTR.fs, tr[1]*pbTR.fs], dtype=np.int64)
    else:
        raise ValueError('unexpected pbTR = {}'.format(pbTR))
    if pbTR.wallTime not in [None]:
        wt = pbTR.wallTime.ToDatetime()
        if wt.year == 0:
            wt = datetime.datetime.min
    else:
        wt = datetime.datetime.min
    return make_time_range(timestamp=ts, fs=pbTR.fs, walltime=wt)


def sig_sel_to_protobuf(sig_sel: SigSelect) -> common_pb2.SigSelector:
    if not isinstance(sig_sel, SigSelect):
        raise ValueError('argument must be a SigSelect')
    if sig_sel.key_idx not in [KeyIndex.NTV]:
        raise ValueError('sig_sel must use KeyIndex.NTV as the key index')
    return common_pb2.SigSelector(ampNtvIdxs=sig_sel.amp.tolist(),
                                  ainNtvIdxs=sig_sel.gpio_ain.tolist(),
                                  dinNtvIdxs=sig_sel.gpio_din.tolist(),
                                  doutNtvIdxs=sig_sel.gpio_dout.tolist())


def make_signal_selector(amp: Union[list, np.array] = None, ain: Union[list, np.array] = None, din: Union[list, np.array] = None, dout: Union[list, np.array] = None) -> SigSelect:
    amp = np.array([], dtype=np.int64) if amp is None else np.array(
        amp, dtype=np.int64)
    ain = np.array([], dtype=np.int64) if ain is None else np.array(
        ain, dtype=np.int64)
    din = np.array([], dtype=np.int64) if din is None else np.array(
        din, dtype=np.int64)
    dout = np.array([], dtype=np.int64) if dout is None else np.array(
        dout, dtype=np.int64)
    return SigSelect(amp=amp, gpio_ain=ain, gpio_din=din, gpio_dout=dout, key_idx=KeyIndex.NTV)


def make_neighbors_desc(nbr: biointerface_pb2.NeighborDescriptor):
    if not isinstance(nbr, biointerface_pb2.NeighborDescriptor):
        raise ValueError('arg must be pb.NeighborDescriptor')
    return NeighborsDesc(radius_um=nbr.radius, ntv_idxs=nbr.ntvChanIdx, distance_um=nbr.distance, theta_deg=nbr.thetaDeg,
                         phi_deg=nbr.phiDeg, N=nbr.num, pos=make_site_pos(nbr.SiteCenterUm, nbr.SiteCenterTcsUm))


def make_neuron_desc(msg: biointerface_pb2.NeuronDescriptor):
    if not isinstance(msg, biointerface_pb2.NeuronDescriptor):
        raise ValueError('arg must be pb.NeuronDescriptor')
    return NeuronDesc(id=msg.neuronID, ntv_idx=msg.siteNtvChanIdx, label=msg.spikeLabel,
                      pos=make_site_pos(msg.posProbe, msg.posTissue), spike_count=msg.spikeCount,
                      spike_rate=msg.spikeRate, mean_abs_peak_waveform=msg.meanAbsPeakWfm, snr=msg.snr,
                      metadata=NeuronExtendedMetadata(neuron_uid=msg.metaData.neuronCatalogUID,
                                                      ensemble_uid=msg.metaData.ensembleUID,
                                                      func_assembly_uid=msg.metaData.funcAssemblyUID,
                                                      dataset_uid=msg.metaData.datasetUID,
                                                      probe_uid=msg.metaData.probeUID))


def from_dense_matrix(mtx: common_pb2.DenseMatrix) -> np.ndarray:
    return np.reshape(np.frombuffer(mtx.data, dtype=np.float64), (mtx.rows, mtx.cols))


class Histogram():
    '''
    Container for histogram data sets.
    '''

    def __init__(self, msg: common_pb2.Histogram):
        if not isinstance(msg, common_pb2.Histogram):
            raise ValueError('msg wrong type')
        self._stats = np.array(msg.stats, dtype=np.float64)
        self._counts = np.array(msg.counts, dtype=np.float64)
        self._dividers = np.array(msg.dividers, dtype=np.float64)
        self._num_bins = np.array(msg.numBins, dtype=np.int64)
        self._is_pdf = msg.numBins

    @property
    def stats(self):
        return self._stats

    @property
    def counts(self):
        return self._counts

    @property
    def dividers(self):
        return self._dividers

    @property
    def num_bins(self):
        return self._num_bins

    @property
    def scale(self):
        return 'pdf' if self._is_pdf else 'counts'


def ntv_to_dset_dict(idxs: KeyIdxs) -> dict[int, int]:
    return {n: d for n, d in zip(idxs.ntv, idxs.dset)}


def dset_to_ntv_dict(idxs: KeyIdxs) -> dict[int, int]:
    return {d: n for n, d in zip(idxs.ntv, idxs.dset)}


def generic_repr(self) -> str:
    class_name = self.__class__.__name__
    fields = [(f, type(getattr(self, f)).__name__) for f in dir(
        self) if not f.startswith('_') and not callable(getattr(self, f))]
    df = pd.DataFrame(fields, columns=['Field', 'Type'])
    df_str = df.to_string(index=False)
    header, rows = df_str.split('\n', 1)
    header_line = '-' * len(header)
    formatted_str = f'{header}\n{header_line}\n{rows}'
    return f'{class_name} with attribues: \n{formatted_str}'


def equal_key_idxs(key_idxs: list[KeyIdxs]) -> bool:
    if len(key_idxs) == 0:
        return False
    ntv = key_idxs[0].ntv
    dset = key_idxs[0].dset
    sys = key_idxs[0].sys
    for k in key_idxs[1:]:
        if k.ntv != ntv or k.dset != dset or k.sys != sys:
            return False
    return True


def is_concatable_trs(trs: list[TimeRange]) -> bool:
    """
    Checks if a list of TimeRange objects are concatable based on their fs and timestamp fields.
    """
    if len(trs) == 0:
        return False
    fs = trs[0].fs
    last_end = trs[0].timestamp[1]
    for tr in trs[1:]:
        if tr.fs != fs:
            return False
        if tr.timestamp[0] != last_end:
            return False
        last_end = tr.timestamp[1]
    return True


def time_range_chunks(tr: TimeRange, num_chunks: int) -> list[TimeRange]:
    """
    Divides a TimeRange object into a list of TimeRange objects based on the number of chunks.
    """
    if num_chunks < 1:
        raise ValueError('num_chunks must be >= 1')
    if num_chunks == 1:
        return [tr]
    total_samps = tr.timestamp[1] - tr.timestamp[0]
    num_samps_per_chunk = int(total_samps / num_chunks)
    start_read = tr.timestamp[0]
    trs = []
    for i in range(num_chunks):
        chunk_time_range = make_time_range(timestamp=[
            start_read,
            min(start_read + num_samps_per_chunk, tr.timestamp[1])
        ], fs=tr.fs)
        trs.append(chunk_time_range)
        start_read += num_samps_per_chunk
    return trs


def calc_num_chans(sig_sel: SigSelect) -> int:
    return len(sig_sel.amp) + len(sig_sel.gpio_ain) + len(sig_sel.gpio_din) + len(sig_sel.gpio_dout)


def calc_sig_size_bytes(sig_sel: SigSelect, tr: TimeRange) -> int:
    return calc_num_chans(sig_sel) * (tr.timestamp[1] - tr.timestamp[0]) * BYTES_PER_SAMPLE




from pathlib import Path

import grpc
import numpy as np
import pandas as pd
from radiens.api.api_utils.util import to_suffix
from radiens.exceptions.grpc_error import handle_grpc_error
from radiens.grpc_radiens import (biointerface_pb2, common_pb2, datasource_pb2,
                                  radiens_dev_pb2, radiensserver_pb2_grpc,
                                  spikesorter_pb2)
from radiens.lib.dataset_metadata import DatasetMetadata
from radiens.lib.sig_metrics import SignalMetrics, SignalMetricsStatus
from radiens.lib.signals_snapshot import PSD, Signals
from radiens.utils.config import new_server_channel
from radiens.utils.constants import TimeRange
from radiens.utils.enums import ClientType, RadiensFileType, RasterMode


def set_datasource(addr, req: datasource_pb2.DataSourceSetSaveRequest):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            res = stub.SetDataSourceFromFile(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        spike_files = []
        for f in res.associatedDsrcs:
            if RadiensFileType.parse(f.fileType) == RadiensFileType.SPIKES:
                spike_files.append(
                    Path(f.path, f.baseName+to_suffix(RadiensFileType.SPIKES)))
        return DatasetMetadata(res), spike_files


def unlink_datasource(addr, datasetIDs: list):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            res = stub.ClearDataSource(
                datasource_pb2.DataSourceRequest(dsourceID=datasetIDs)
            )
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        return list(res.sortedID)


def list_datasource_ids(addr):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            res = stub.ListDataSourceIDs(datasource_pb2.DataSourceRequest())
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        return list(res.sortedID)


def convert_kilosort_output(addr, req: radiens_dev_pb2.WrangleRequest):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensSpikeSorter1Stub(chan)
        try:
            stub.SpikeSorterWrangleData(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        return


def get_signals(addr, req):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            raw = stub.GetHDSnapshotPy(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        return Signals(raw)


def get_psd(addr, req):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            resp = stub.GetPSD(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        if req.isReturnPSD:
            return PSD(resp)


def get_spike_waveforms(addr, dsource_id: str, time_range: list):
    if len(time_range) != 2:
        raise ValueError("time range must be a list of length 2")
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            req = biointerface_pb2.SpikesGetSpikesRequest(spikeLabels=[])
            req.dsourceID = dsource_id
            req.mode = biointerface_pb2.NeuronsSignalMode.SpikeWaveforms
            req.timeStart = time_range[0]
            req.timeStop = time_range[1]
            req.maxSpikesPerChannel = 1000  # arbitrary for now (TODO)

            raw = stub.SpikesGetSpikesDense(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)

        _waveforms = np.reshape(
            np.frombuffer(raw.data, dtype=np.float32),
            (raw.totalNSpikes, raw.waveformNPoints),
        )

        return _waveforms


def get_spikes_timestamps(addr, dsource_id: str, time_range: TimeRange):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            req = spikesorter_pb2.SpikeSorterGetRasterDataRequest(
                spikeSorterID=dsource_id,
                timeWindow=1,
                # fs * timeWindow/plotWidthPoints = sampFactor (1)
                plotWidthPoints=time_range.fs,
                componentID="",
                mode=RasterMode.CHANNELS.pb_value,
                timeRange=time_range.sec,
                labeledOnly=True,
                primarySiteOnly=True,
            )

            raw = stub.SpikesGetRasterData(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)

        timestamps = np.empty_like(raw.spikeTimestamps, dtype=np.float64)
        labels = np.empty_like(raw.spikeTimestamps, dtype=np.int32)
        dset_idxs = np.empty_like(raw.spikeTimestamps, dtype=np.int32)

        for (i, msg) in enumerate(raw.spikeTimestamps):
            timestamps[i] = msg.spikeTimestamp
            labels[i] = msg.label
            dset_idxs[i] = msg.dataIdxs[0]
        return {"timestamps": timestamps, "labels": labels, "dset_idxs": dset_idxs}


# ==========================
# ====== KPIs ==============
# ==========================


def set_kpi_calculate(addr, req) -> None:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            stub.KpiCalculate(common_pb2.KpiStandardRequest(req))
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)


def set_kpi_clear(addr, dsource_id) -> None:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            stub.KpiClear(common_pb2.KpiStandardRequest(
                dsourceID=[dsource_id]))
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)


def set_kpi_params(addr, req):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            stub.SetKpiParam(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)


def set_kpi_packet_dur(addr, req):
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            stub.SetKpiPacketDur(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)


def get_kpi_metrics(addr, dsource_id, req) -> SignalMetrics:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            resp = stub.KpiGetMetrics(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        return SignalMetrics(dsource_id, resp)


def get_kpi_status(addr, dsource_id) -> SignalMetricsStatus:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            resp = stub.GetKpiStatus(
                common_pb2.GetKpiStatusRequest(streamGroupId=dsource_id)
            )
            resp2 = stub.GetDataSourceKpiFileStatus2(
                datasource_pb2.DataSourceRequest(dsourceID=[dsource_id])
            )
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        return SignalMetricsStatus(resp, resp2)


def get_kpi_params(addr, dsource_id) -> pd.DataFrame:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.RadiensCoreStub(chan)
        try:
            resp = stub.GetKpiParam(
                datasource_pb2.DataSourceRequest(dsourceID=[dsource_id])
            )
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        p = {
            "ntv_idx": [],
            "chan_enabled": [],
            "thr_activated": [],
            "thr": [],
            "thr_sd": [],
            "thr_wdw": [],
            "shadow": [],
            "weak_thr_activated": [],
            "weak_thr": [],
            "weak_thr_sd": [],
            "thr_wdw_pts": [],
            "shadow_pts": [],
        }
        for v in resp.rec:
            p["ntv_idx"].append(v.ntvChanIdx)
            p["chan_enabled"].append(v.isEnabled)
            p["thr_activated"].append(v.isSetThr)
            p["thr"].append(np.array(v.thr))
            p["thr_sd"].append(np.array(v.thrSd))
            p["thr_wdw"].append(np.around(np.array(v.thrWdw) * 1000.0, 5))
            p["shadow"].append(np.around(np.array(v.shadow) * 1000.0, 5))
            p["weak_thr_activated"].append(np.array(v.isSetWeakThr))
            p["weak_thr"].append(np.array(v.weakThr))
            p["weak_thr_sd"].append(np.array(v.weakThrSd))
            p["thr_wdw_pts"].append(v.thrWdwPts)
            p["shadow_pts"].append(v.shadowPts)
        return pd.DataFrame(p)


def command_dashboard(addr, req) -> str:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.DashboardsStub(chan)
        try:
            stub.CommandDashboard(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)


def summa_list_sessions(addr, req) -> str:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.DashboardsStub(chan)
        try:
            resp = stub.ListSessions(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        d = {}
        for session in resp.session:
            d[session.summaID] = {"dsource_ids": session.dsourceIDs}
        return d


def summa_clear_sessions(addr, req) -> str:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.DashboardsStub(chan)
        try:
            stub.ClearAllSessions(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)


def summa_launch_session_analysis(addr, req) -> str:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.DashboardsStub(chan)
        try:
            resp = stub.LaunchSessionAnalysis(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        return resp


def summa_get_session_analysis(addr, req) -> str:
    with new_server_channel(addr) as chan:
        stub = radiensserver_pb2_grpc.DashboardsStub(chan)
        try:
            resp = stub.GetSessionAnalysis(req)
        except grpc.RpcError as ex:
            handle_grpc_error(ex, ClientType.VIDERE)
        return resp



import time
import unittest
from pathlib import Path

from radiens.allego_client import AllegoClient
from radiens.lib.spikes import SpikesSet
from radiens.utils.enums import SignalType


class base_utest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls._dir = Path(__file__).parent

    def setUp(self):
        self.client = AllegoClient()
        self.client.set_streaming('off')
        self.client.spike_sorter().set_sorting('off')
        self.client.set_sampling_freq(30000)
        self.client.spike_sorter().initialize()
        time.sleep(1.0)

    def test_get_status(self):
        status = self.client.get_status()
        self.assertEqual('sim-spikes', status.system_mode)
        self.assertEqual(30000, status.sample_freq)

    def test_set_dio(self):
        self.client.set_digital_out_manual(True, False)
        reg = self.client.get_digital_out_states()
        self.assertEqual('manual', reg['digital_outs_mode'])
        self.assertTrue(reg['states'][0]['state'])
        self.assertEqual(0, reg['states'][0]['chan_idx'])
        self.assertFalse(reg['states'][1]['state'])
        self.assertEqual(1, reg['states'][1]['chan_idx'])

    def test_get_channel_metadata(self):
        meta = self.client.get_channel_metadata()
        self.assertEqual(32, meta.num_sigs(SignalType.AMP))
        self.assertEqual(2, meta.num_sigs(SignalType.AIN))
        self.assertEqual(2, meta.num_sigs(SignalType.DIN))
        self.assertEqual(2, meta.num_sigs(SignalType.DOUT))

    def test_signal_metrics(self):
        resp = self.client.signal_metrics().get_metrics_status()
        self.assertEqual(0.2, resp.attributes['packet_dur_sec'])
        self.client.set_streaming('on')
        time.sleep(2.0)
        resp = self.client.signal_metrics().get_metrics_status()
        self.assertGreater(resp.time_range.sec[1],  0)
        resp.print()
        resp = self.client.signal_metrics().get_metrics(3)
        resp.print()
        print("val = ", resp.val)
        self.client.set_streaming('off')

    def test_streaming_and_spike_sorting(self):
        self.client.spike_sorter().initialize()
        time.sleep(3.0)
        status = self.client.get_status()
        self.assertEqual('S_OFF', status.stream.mode)
        dash = self.client.spike_sorter().get_dashboard()
        self.assertEqual('SYS_OFF', dash.state.loc['mode'][0])
        self.client.set_streaming('on')
        time.sleep(0.5)
        status = self.client.get_status()
        self.assertEqual('S_ON', status.stream.mode)
        resp = self.client.spike_sorter().get_state()
        self.assertEqual('SYS_OFF', resp.state.loc['mode'][0])
        self.assertFalse(resp.state.loc['is_on'][0])
        self.client.spike_sorter().set_sorting('on')
        time.sleep(3.0)
        resp = self.client.spike_sorter().get_state()
        self.assertEqual('SYS_ON', resp.state.loc['mode'][0])
        self.assertTrue(resp.state.loc['is_on'][0])
        dash = self.client.spike_sorter().get_dashboard()
        self.assertEqual('SYS_ON', dash.state.loc['mode'][0])
        self.client.spike_sorter().set_sorting('off')
        time.sleep(2.0)
        resp = self.client.spike_sorter().get_state()
        self.assertEqual('SYS_OFF', resp.state.loc['mode'][0])
        self.assertFalse(resp.state.loc['is_on'][0])
        dash = self.client.spike_sorter().get_dashboard()
        self.assertEqual('SYS_OFF', dash.state.loc['mode'][0])
        self.client.set_streaming('off')
        time.sleep(0.25)
        status = self.client.get_status()
        self.assertEqual('S_OFF', status.stream.mode)

        resp = self.client.spikes().get_recent_spikes()
        self.assertTrue(isinstance(resp, SpikesSet))
        self.assertEqual(67, resp.num_pts_waveform)
        self.assertGreater(resp.num_spikes_total, 0)
        self.assertTrue(resp.has_component('waveforms'))
        self.assertFalse(resp.has_component('timestamps'))

        df_params = self.client.spike_sorter().get_params()
        thr_level = df_params.loc[df_params['ntv_idx'] == 0]['thr'].to_numpy()[
            0]
        self.assertEqual(80, thr_level[0])
        self.assertEqual(120, thr_level[1])

        self.client.spike_sorter().set_threshold_level(neg_thr=523, pos_thr=469)
        df_params = self.client.spike_sorter().get_params()
        thr_level = df_params.loc[df_params['ntv_idx'] == 0]['thr'].to_numpy()[
            0]
        self.assertEqual(523, thr_level[0])
        self.assertEqual(469, thr_level[1])

        self.client.spike_sorter().set_threshold_level(neg_thr=80, pos_thr=120)
        df_params = self.client.spike_sorter().get_params()
        thr_level = df_params.loc[df_params['ntv_idx'] == 0]['thr'].to_numpy()[
            0]
        self.assertEqual(80, thr_level[0])
        self.assertEqual(120, thr_level[1])

        spec = self.client.spikes().get_spikes_metadata()
        self.assertEqual(0, spec.time_range[0][0])
        self.assertGreater(spec.time_range[0][1], 0)
        self.assertEqual(7, len(spec.site_summary_stats))
        self.assertEqual(32, len(spec.sites))
        self.assertGreater(len(spec.neurons), 0)


if __name__ == "__main__":
    unittest.main()



"""
radiens functional testing CLI
"""

import multiprocessing
import os
import subprocess
from pathlib import Path

import click
# https://matplotlib.org/stable/users/explain/backends.html
from click_shell import shell
from radiens.allego_client import AllegoClient
from radiens.cli.allego import allego
from radiens.cli.file_sys import filesys
from radiens.cli.sys import system
from radiens.cli.videre import videre
from radiens.file_sys_client import FileSystemClient
from radiens.videre_client import VidereClient


@shell(prompt='radiens > ', intro='Starting RADIENS utility (use `> help usage` for usage information)')
@ click.pass_context
def cli(ctx):
    ctx.ensure_object(dict)
    if 'fsys_client' not in ctx.obj:
        ctx.obj['fsys_client'] = FileSystemClient()
        ctx.obj['allego_client'] = AllegoClient()
        ctx.obj['videre_client'] = VidereClient()
        ctx.obj['cmd_app'] = ''


@ cli.command(short_help='usage information')
@ click.pass_context
def usage(ctx):
    ''' RADIENS CLI utility

        Notes:
        (1) See `readme.md` for instructions.
    '''


@ cli.command('os', short_help='run OS command')
@ click.argument('cmd', nargs=-1)
@ click.pass_context
def os_cmd(ctx, cmd):
    ''' Shell out to run OS command.

        If the command line includes flags, then wrap `cmd` in quotes. 
    '''
    if len(cmd) == 0:
        return
    if cmd[0] in ['cd']:
        if len(cmd) == 2:
            os.chdir(Path(cmd[1]).expanduser().absolute())
        subprocess.run('pwd', shell=True)
        return
    subprocess.run(' '.join(cmd), shell=True)


cli.add_command(filesys)
cli.add_command(system)
cli.add_command(allego)
cli.add_command(videre)

if __name__ == '__main__':
    multiprocessing.freeze_support()
    cli(obj={})



from collections import namedtuple
from pathlib import Path
from pprint import pprint
from typing import Dict

import pandas as pd
from radiens.grpc_radiens import allegoserver_pb2, common_pb2, datasource_pb2
from radiens.lib.dataset_metadata import FileSetDescriptor
from radiens.utils.enums import Port, SystemMode

PORT_SPEC = namedtuple("PortSpec", ['port_num_chans', 'probe_name', 'probe_num_sites',
                                    'headstage_name', 'headstage_num_chans'])

STREAMING_SPEC = namedtuple(
    "StreamSpec", ['mode', 'time_range', 'hw_mem_level'])

RECORDING_SPEC = namedtuple("RecordingSpec", ['mode', 'file_name', 'dur_sec', 'error',
                                              'path', 'base_name', 'index', 'timestamp'])
GPIO_SPEC = namedtuple("GPIOSpec", ['ain', 'din', 'dout'])

STATUS_SPEC = namedtuple("StatusSpec", ['streaming', 'recording', 'ports',
                                        'connected', 'gpio', 'sample_freq', 'mode'])


class AllegoState():
    '''
    Container for Allego state.
    '''

    def __init__(self, raw=allegoserver_pb2.ConfigAndStatus):
        self._stream = STREAMING_SPEC(mode=allegoserver_pb2.StreamMode.Name(raw.streaming.streamMode),
                                      time_range=raw.streaming.primaryCacheTRange,
                                      hw_mem_level=raw.streaming.hardwareMemoryLevel)
        _path = str(
            Path(raw.recording.activeFileName).expanduser().absolute().parent)
        _base_name = Path(Path(raw.recording.activeFileName)).stem
        _idx = -1
        _tstamp = ''
        if '__uid' in _base_name and len(_base_name) > _base_name.index('__uid')+6:
            uid_idx = _base_name.index('__uid')
            _tstamp = _base_name[uid_idx+5:]
            _idx_idx = _base_name[:uid_idx].rfind('_')
            if _idx_idx > 0:
                _idx = int(_base_name[_idx_idx+1:uid_idx])
                _base_name = _base_name[:_idx_idx]
        elif '_' in _base_name:
            _idx_idx = _base_name.rfind('_')
            if _idx_idx > 0 and len(_base_name) > _idx_idx+1:
                try:
                    _idx = int(_base_name[_idx_idx+1:])
                except ValueError:
                    _idx = -1
                else:
                    _base_name = _base_name[:_idx_idx]
        self._recording = RECORDING_SPEC(mode=allegoserver_pb2.RecordMode.Name(raw.recording.recordMode),
                                         file_name=raw.recording.activeFileName,
                                         dur_sec=raw.recording.duration,
                                         error=raw.recording.error,
                                         base_name=_base_name,
                                         index=_idx,
                                         timestamp=_tstamp,
                                         path=_path)
        self._port_spec = {}
        for k, port in enumerate(raw.ports):
            self._port_spec[Port.parse(port.port)] = port.channelCount
        self._is_connected = raw.isConnected
        self._gpio_spec = GPIO_SPEC(
            ain=raw.gpioChannelCount.nAux, din=raw.gpioChannelCount.nDin, dout=raw.gpioChannelCount.nDout)
        self._fs = raw.baseSampFreq
        self._cable_lengths = {}
        for p in Port.all():
            self._cable_lengths[p] = eval('raw.cableLengths.{}'.format(p.name))
        self._backbone_mode = SystemMode.parse(raw.backboneMode)

    @ property
    def sample_freq(self) -> float:
        """
        Current system sample frequency in samples/sec [Hz]
        """
        return self._fs

    @ property
    def system_mode(self) -> SystemMode:
        """
        Current Allego mode, e.g. 'sim-spikes', etc.
        """
        return self._backbone_mode

    @ property
    def port_num_channels(self) -> Dict[Port, int]:
        """
        Number of channels on each connected port.
        """
        return self._port_spec

    @ property
    def cable_length_ft(self) -> Dict[Port, int]:
        """
        Headstage cable length in feet for each system port
        """
        return self._cable_lengths

    @ property
    def recording(self) -> RECORDING_SPEC:
        """
        Recording status
        """
        return self._recording

    @ property
    def stream(self) -> STREAMING_SPEC:
        """
        Streaming status
        """
        return self._stream

    @ property
    def connected(self) -> bool:
        """
        Returns true if there is a headstage connected on any channel.
        """
        return self._is_connected

    @ property
    def base_name(self) -> str:
        """
        Returns true if there is a headstage connected on any channel.
        """

    def print(self, detail: str = 'brief'):
        if detail not in ['brief', 'verbose']:
            raise ValueError('detail must be [`brief`, `verbose`]')
        print('ALLEGO status:')
        if self.connected in [False]:
            print('not connected')
            return
        if detail in ['brief', 'verbose']:
            print('stream       : {}'.format(self.stream.mode))
            print('  time range : [0.000, {:.3f}] sec'.format(
                self.stream.time_range[1]))
            print('  HW memory  : {} '.format(self.stream.hw_mem_level))
            print('recording         : {}'.format(self.recording.mode))
            print('  duration        : {:.3f} sec'.format(
                self.recording.dur_sec))
            print('  file name       : {}'.format(
                Path(self.recording.file_name).stem))
            print('  base name       : {}'.format(self.recording.base_name))
            print('  path            : {}'.format(self.recording.path))
            print('  index           : {}'.format(self.recording.index))
            print('  file timestamp  : {}'.format(self.recording.timestamp))
        if detail in ['verbose']:
            print('settings   ')
            print('  sample freq   : {:.0f}'.format(self.sample_freq))
            print('  port channels : {}'.format(
                Port.map_into_keys_as_names(self.port_num_channels)))
            print('  cable lengths : {}'.format(
                Port.map_into_keys_as_names(self.cable_length_ft)))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: spikesorter.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from . import biointerface_pb2 as biointerface__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11spikesorter.proto\x12\x06\x61llego\x1a\x0c\x63ommon.proto\x1a\x12\x62iointerface.proto\"\xaa\x02\n\x18SpikeSorterLaunchRequest\x12\x11\n\tdsourceID\x18\x01 \x01(\t\x12&\n\x06source\x18\x02 \x01(\x0b\x32\x16.allego.FileDescriptor\x12\x42\n\x0f\x63lustererParams\x18\x03 \x01(\x0b\x32).allego.SpikeSorterClustererParamsRequest\x12>\n\rfeatureParams\x18\x04 \x01(\x0b\x32\'.allego.SpikeSorterFeatureParamsRequest\x12\x15\n\rdiscoverNoise\x18\x05 \x01(\x08\x12\x10\n\x08isAutoOn\x18\x06 \x01(\x08\x12\x12\n\nnbrPattern\x18\x07 \x01(\x05\x12\x12\n\nsinkDsrcID\x18\x08 \x01(\t\"7\n\x1dSpikeSorterIsAnyActiveRequest\x12\x16\n\x0espikeSorterIDs\x18\x01 \x03(\t\"O\n\x1bSpikeSorterIsAnyActiveReply\x12\x13\n\x0bisAnyActive\x18\x01 \x02(\x08\x12\x1b\n\x13\x61\x63tiveSpikeSorterID\x18\x02 \x01(\t\"\x8e\x02\n\x19SpikeSorterCommandRequest\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12\'\n\x03\x63md\x18\x02 \x02(\x0e\x32\x1a.allego.SpikeSorterCommand\x12-\n\x06subCmd\x18\x03 \x02(\x0e\x32\x1d.allego.SpikeSorterSubCommand\x12\x42\n\x0f\x63lustererParams\x18\x04 \x01(\x0b\x32).allego.SpikeSorterClustererParamsRequest\x12>\n\rfeatureParams\x18\x05 \x01(\x0b\x32\'.allego.SpikeSorterFeatureParamsRequest\"\x93\x01\n\x10SpikeSorterState\x12+\n\x03sys\x18\x01 \x02(\x0e\x32\x1e.allego.SpikeSorterStateSystem\x12\x14\n\x0c\x66racComplete\x18\x02 \x01(\x01\x12\x0b\n\x03msg\x18\x03 \x01(\t\x12\x0c\n\x04isOn\x18\x08 \x01(\x08\x12\x10\n\x08\x65rrorMsg\x18\t \x01(\t\x12\x0f\n\x07warnMsg\x18\n \x01(\t\"\x81\x01\n\x0cSpkSortIOreq\x12#\n\x04type\x18\x01 \x02(\x0e\x32\x15.allego.SpkSortIOtype\x12\'\n\x06target\x18\x02 \x02(\x0e\x32\x17.allego.SpkSortIOtarget\x12#\n\x04mode\x18\x03 \x02(\x0e\x32\x15.allego.SpkSortIOmode\"\x8f\x01\n\x16SpikeSorterLaunchReply\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12+\n\nsorterType\x18\x02 \x02(\x0e\x32\x17.allego.SpikeSorterType\x12\x0b\n\x03msg\x18\x03 \x01(\t\x12$\n\x04\x64\x65sc\x18\x04 \x01(\x0b\x32\x16.allego.FileDescriptor\"1\n\x18GetSpikeSorterIDsRequest\x12\x15\n\rstreamGroupID\x18\x01 \x02(\t\"0\n\x16GetSpikeSorterIDsReply\x12\x16\n\x0espikeSorterIDs\x18\x01 \x03(\t\"\xdd\x01\n\x1fSpikeSorterGetRasterDataRequest\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12\x12\n\ntimeWindow\x18\x03 \x02(\x01\x12\x17\n\x0fplotWidthPoints\x18\x04 \x02(\x01\x12\x13\n\x0b\x63omponentID\x18\x05 \x02(\t\x12\x11\n\ttimeRange\x18\x06 \x03(\x01\x12 \n\x04mode\x18\x07 \x02(\x0e\x32\x12.allego.RasterMode\x12\x13\n\x0blabeledOnly\x18\x08 \x01(\x08\x12\x17\n\x0fprimarySiteOnly\x18\t \x01(\x08\"\x9d\x01\n\x1aSpikeSorterRasterDataReply\x12/\n\x0fspikeTimestamps\x18\x02 \x03(\x0b\x32\x16.allego.SpikeTimestamp\x12\x11\n\ttimeRange\x18\x03 \x03(\x01\x12\x16\n\x0etimeStampRange\x18\x04 \x03(\x03\x12\x10\n\x08GPIOData\x18\x05 \x02(\x0c\x12\x11\n\tGPIOShape\x18\x06 \x03(\x05\"J\n\x1aSpikeSorterStandardRequest\x12\x15\n\rstreamGroupID\x18\x01 \x01(\t\x12\x15\n\rspikeSorterID\x18\x02 \x01(\t\"_\n\x1bSpikeSorterDashboardRequest\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12)\n\x0c\x64\x61shElements\x18\x02 \x03(\x0e\x32\x13.allego.DashElement\"\x86\x03\n\x12SpikeSorterDynamic\x12\x11\n\tisEnabled\x18\x01 \x01(\x08\x12\x43\n\x0fupdatePeriodSec\x18\x02 \x01(\x0b\x32*.allego.SpikeSorterDynamic.UpdatePeriodSec\x12\x35\n\x08\x63riteria\x18\x03 \x01(\x0b\x32#.allego.SpikeSorterDynamic.Criteria\x1a\xa0\x01\n\x08\x43riteria\x12\x31\n\x06\x64\x65tect\x18\x01 \x01(\x0e\x32!.allego.SpikeSorterCriterionLevel\x12\x30\n\x05train\x18\x02 \x01(\x0e\x32!.allego.SpikeSorterCriterionLevel\x12/\n\x04sort\x18\x03 \x01(\x0e\x32!.allego.SpikeSorterCriterionLevel\x1a>\n\x0fUpdatePeriodSec\x12\x0e\n\x06\x64\x65tect\x18\x01 \x01(\x01\x12\r\n\x05train\x18\x02 \x01(\x01\x12\x0c\n\x04sort\x18\x03 \x01(\x01\"\xc5\x01\n\x1cSpikeSorterSetDynamicRequest\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12\x42\n\x07variant\x18\x02 \x03(\x0b\x32\x31.allego.SpikeSorterSetDynamicRequest.VariantEntry\x1aJ\n\x0cVariantEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.allego.SpikeSorterDynamic:\x02\x38\x01\"\xd8\x01\n\x1aSpikeSorterGetDynamicReply\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12\x15\n\rrootVariantID\x18\x02 \x02(\t\x12@\n\x07variant\x18\x03 \x03(\x0b\x32/.allego.SpikeSorterGetDynamicReply.VariantEntry\x1aJ\n\x0cVariantEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.allego.SpikeSorterDynamic:\x02\x38\x01\"l\n\x1fGetSpikeSorterParamCommandReply\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12\x32\n\x03rec\x18\x02 \x03(\x0b\x32%.allego.SpikeSorterParamCommandRecord\"3\n\x12\x43lusterMapOutliers\x12\x0c\n\x04mild\x18\x01 \x03(\x01\x12\x0f\n\x07\x65xtreme\x18\x02 \x03(\x01\"9\n\x13\x43lusterMapQuartiles\x12\n\n\x02q1\x18\x01 \x02(\x01\x12\n\n\x02q2\x18\x02 \x02(\x01\x12\n\n\x02q3\x18\x03 \x02(\x01\"\x83\x01\n\x0f\x43lusterMapStats\x12\x16\n\x0e\x63\x65ntroidCenter\x18\x01 \x03(\x01\x12\x12\n\ncentroidSd\x18\x02 \x03(\x01\x12\x16\n\x0e\x63\x65ntroidSdNorm\x18\x03 \x02(\x01\x12\x19\n\x11intraCentroidDist\x18\x04 \x03(\x01\x12\x11\n\tnumSpikes\x18\x05 \x02(\x03\"f\n\x10\x43lusterMapRecord\x12\r\n\x05label\x18\x01 \x02(\x05\x12&\n\x05stats\x18\x03 \x02(\x0b\x32\x17.allego.ClusterMapStats\x12\x1b\n\x13numSpikesTrainCache\x18\x04 \x02(\x03\"\xa8\x01\n\x13SpikeSortSiteStatus\x12\x12\n\nntvChanIdx\x18\x01 \x02(\x05\x12\x15\n\rorderedLabels\x18\x02 \x03(\x05\x12)\n\x07\x63luster\x18\x03 \x03(\x0b\x32\x18.allego.ClusterMapRecord\x12\x11\n\tnumSpikes\x18\x04 \x03(\x03\x12\x0c\n\x04zeta\x18\x05 \x03(\x01\x12\x1a\n\x12\x61ggregateSpikeRate\x18\x06 \x02(\x01\"\x8f\x01\n\x14SpikeSortPhaseStatus\x12\x11\n\ttimeRange\x18\x01 \x03(\x01\x12\x12\n\nchunkIndex\x18\x02 \x02(\x03\x12\x14\n\x0cupdatePeriod\x18\x03 \x02(\x01\x12\x0c\n\x04\x62\x65ta\x18\x04 \x02(\x01\x12\x12\n\nisComplete\x18\x05 \x02(\x08\x12\x18\n\x10\x66ractionComplete\x18\x06 \x02(\x01\"*\n\x13SpikeSortNbrIdxList\x12\x13\n\x0bntvChanIdxs\x18\x01 \x03(\x05\"\xf2\x02\n\x16SpikeSorterStatusReply\x12\'\n\x05state\x18\x01 \x02(\x0b\x32\x18.allego.SpikeSorterState\x12\x16\n\x0e\x62iointerfaceID\x18\x03 \x02(\t\x12\x14\n\x0c\x64\x61tasourceID\x18\x04 \x02(\t\x12\x10\n\x08sampFreq\x18\x05 \x02(\x01\x12\x19\n\x11\x65nabledNtvChanIdx\x18\x06 \x03(\x05\x12+\n\x05phase\x18\x07 \x03(\x0b\x32\x1c.allego.SpikeSortPhaseStatus\x12\x30\n\x0b\x65nabledSite\x18\x08 \x03(\x0b\x32\x1b.allego.SpikeSortSiteStatus\x12\x12\n\nprobeYield\x18\t \x02(\x01\x12\x11\n\tsortStats\x18\n \x03(\x01\x12\x1b\n\x13\x64\x61tasourceTimeRange\x18\x0c \x03(\x01\x12\x1d\n\x15\x62iointerfaceTimeRange\x18\r \x03(\x01\x12\x12\n\nnumNeurons\x18\x0f \x02(\x03\"R\n\x1eSpikeSorterStatusVariantsReply\x12\x30\n\x08variants\x18\x01 \x03(\x0b\x32\x1e.allego.SpikeSorterStatusReply\"\xfd\x02\n\x19SpikeSorterClassifierSpec\x12\r\n\x05\x61lpha\x18\x01 \x02(\x01\x12\x11\n\talphaBias\x18\x02 \x02(\x01\x12\x16\n\x0e\x64imOutputLayer\x18\x03 \x02(\x05\x12\x16\n\x0e\x65rrorTolerance\x18\x04 \x02(\x01\x12\x19\n\x11hiddenLayerFactor\x18\x05 \x02(\x01\x12\n\n\x02iD\x18\x06 \x02(\t\x12\x16\n\x0einitBiasStdDev\x18\x07 \x02(\x01\x12\x18\n\x10initWeightStdDev\x18\x08 \x02(\x01\x12\x11\n\tisMasking\x18\t \x02(\x08\x12\x10\n\x08l1lambda\x18\n \x02(\x01\x12\x15\n\rminibatchSize\x18\x0b \x02(\x05\x12\x15\n\rmaxNumSamples\x18\x0c \x02(\x05\x12\x11\n\tnumEpochs\x18\r \x02(\x05\x12\x19\n\x11numSamplesTestMSE\x18\x0e \x02(\x05\x12\x1b\n\x13numConcurrentModels\x18\x0f \x02(\x05\x12\x17\n\x0flabelNoiseLevel\x18\x10 \x02(\x01\"u\n\x16SpikeSorterVariantSpec\x12\n\n\x02iD\x18\x01 \x02(\t\x12\x14\n\x0csiteConfigID\x18\x02 \x02(\t\x12\x39\n\x0e\x63lassifierSpec\x18\x03 \x02(\x0b\x32!.allego.SpikeSorterClassifierSpec\"\xcc\x01\n\x0fSpikeSorterSpec\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12\x14\n\x0c\x64\x61tasourceID\x18\x02 \x02(\t\x12\x16\n\x0e\x62iointerfaceID\x18\x03 \x02(\t\x12\x1b\n\x13maxNumSiteNeighbors\x18\x04 \x02(\x05\x12\x18\n\x10neighborRadiusUm\x18\x05 \x02(\x01\x12/\n\x07variant\x18\x06 \x03(\x0b\x32\x1e.allego.SpikeSorterVariantSpec\x12\x0c\n\x04seed\x18\x08 \x02(\x03\"\xc2\x01\n\x19SpikeSorterGetConfigReply\x12%\n\x04spec\x18\x01 \x02(\x0b\x32\x17.allego.SpikeSorterSpec\x12\x16\n\x0e\x61IupdatePeriod\x18\x02 \x02(\x01\x12\x19\n\x11orderedVariantIDs\x18\x03 \x03(\t\x12\x15\n\rrootVariantID\x18\x04 \x02(\t\x12\x34\n\x0csorterStatus\x18\x06 \x03(\x0b\x32\x1e.allego.SpikeSorterStatusReply\"\x90\x02\n\x15SpikeSorterVizRequest\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12\x11\n\tvariantID\x18\x02 \x02(\t\x12\x12\n\nntvChanIdx\x18\x03 \x03(\x05\x12\x13\n\x0bspikeLabels\x18\x04 \x03(\x05\x12\x34\n\x07vizType\x18\x05 \x02(\x0e\x32#.allego.SpikeSorterVizRequest.VType\x12\x14\n\x0cisYAutoScale\x18\x06 \x02(\x08\x12\x0c\n\x04yLim\x18\x07 \x03(\x01\x12\x0e\n\x06\x66igDir\x18\x08 \x02(\t\x12\x11\n\tnumSpikes\x18\t \x02(\x05\"\'\n\x05VType\x12\x1e\n\x1aVIZ_SPIKESORT_TRAIN_SPIKES\x10\x00\"\x8b\x01\n!SpikeSorterClustererParamsRequest\x12\x18\n\x10posInfluenceFrac\x18\x01 \x01(\x01\x12\x1d\n\x15wfmShapeInfluenceFrac\x18\x02 \x01(\x01\x12\x0b\n\x03\x65ps\x18\x03 \x01(\x01\x12\x0e\n\x06minPts\x18\x04 \x01(\x05\x12\x10\n\x08\x64isabled\x18\x05 \x01(\x08\"\x90\x01\n\x1fSpikeSorterFeatureParamsRequest\x12\x13\n\x0bnumFeatures\x18\x01 \x02(\x05\x12-\n\x0b\x66\x65\x61tureType\x18\x02 \x01(\x0e\x32\x18.allego.InputFeatureType\x12\x11\n\tlatentDim\x18\x03 \x01(\x05\x12\x16\n\x0eresampleWfmLen\x18\x04 \x01(\x05\"\xe9\x01\n\x1dSpikeSorterFeatureParamsReply\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12-\n\x0b\x66\x65\x61tureType\x18\x02 \x02(\x0e\x32\x18.allego.InputFeatureType\x12\x16\n\x0emaxNumFeatures\x18\x03 \x02(\x05\x12\x13\n\x0bnumFeatures\x18\x04 \x02(\x05\x12\x18\n\x10posInfluenceFrac\x18\x05 \x02(\x02\x12\x1d\n\x15wfmShapeInfluenceFrac\x18\x06 \x02(\x02\x12\x1c\n\x14isClusteringDisabled\x18\x07 \x02(\x08\"\xb6\x05\n\x19SpikeSorterDashboardReply\x12\"\n\x0c\x65nabledPorts\x18\x01 \x03(\x0e\x32\x0c.allego.Port\x12\x37\n\x07general\x18\x02 \x01(\x0b\x32&.allego.SpikeSorterDashboardGeneralRec\x12\x33\n\tsiteStats\x18\x03 \x01(\x0b\x32 .allego.IndicoDashboardSiteStats\x12\x43\n\tportStats\x18\x04 \x03(\x0b\x32\x30.allego.SpikeSorterDashboardReply.PortStatsEntry\x12\x43\n\tsitePanel\x18\x05 \x03(\x0b\x32\x30.allego.SpikeSorterDashboardReply.SitePanelEntry\x12G\n\x0bneuronPanel\x18\x06 \x03(\x0b\x32\x32.allego.SpikeSorterDashboardReply.NeuronPanelEntry\x12*\n\x02\x61I\x18\x07 \x01(\x0b\x32\x1e.allego.SpikeSorterDashboardAI\x1aR\n\x0ePortStatsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12/\n\x05value\x18\x02 \x01(\x0b\x32 .allego.IndicoDashboardPortStats:\x02\x38\x01\x1aW\n\x0eSitePanelEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.allego.IndicoDashboardSiteIndicators:\x02\x38\x01\x1a[\n\x10NeuronPanelEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x36\n\x05value\x18\x02 \x01(\x0b\x32\'.allego.IndicoDashboardNeuronIndicators:\x02\x38\x01\"\xac\x03\n\x1eSpikeSorterDashboardGeneralRec\x12\'\n\x05state\x18\x01 \x02(\x0b\x32\x18.allego.SpikeSorterState\x12+\n\nsorterType\x18\x02 \x02(\x0e\x32\x17.allego.SpikeSorterType\x12\x10\n\x08sorterID\x18\x03 \x02(\t\x12\x14\n\x0c\x64\x61tasourceID\x18\x04 \x02(\t\x12\x11\n\ttimeRange\x18\x05 \x03(\x01\x12\x15\n\rnumTotalSites\x18\x06 \x02(\x03\x12\x17\n\x0fnumEnabledSites\x18\x07 \x02(\x03\x12\x16\n\x0enumActiveSites\x18\x08 \x02(\x03\x12\x12\n\nnumNeurons\x18\t \x02(\x03\x12\x12\n\nprobeYield\x18\n \x02(\x01\x12\x11\n\tsiteYield\x18\x0b \x02(\x01\x12\x1a\n\x12numSpikesProcessed\x18\x0c \x02(\x03\x12\x18\n\x10numSpikesLabeled\x18\r \x02(\x03\x12\x16\n\x0esortEfficiency\x18\x0e \x02(\x01\x12(\n\x08sinkDesc\x18\x0f \x02(\x0b\x32\x16.allego.FileDescriptor\"\xd5\x01\n\x16SpikeSorterDashboardAI\x12\x0c\n\x04\x62\x65ta\x18\x01 \x02(\x01\x12\"\n\x1a\x65stimatedReaderCompleteSec\x18\x02 \x02(\x01\x12\x1a\n\x12interSessionDurSec\x18\x03 \x02(\x01\x12\x19\n\x11interSessionStart\x18\x04 \x02(\t\x12\x15\n\rsessionDurSec\x18\x05 \x02(\x01\x12\x14\n\x0csessionStart\x18\x06 \x02(\t\x12\x12\n\nsessionIdx\x18\x07 \x02(\x03\x12\x11\n\tstartTime\x18\x08 \x02(\t\"_\n\x18IndicoDashboardSiteStats\x12\x0b\n\x03snr\x18\x01 \x03(\x01\x12\r\n\x05noise\x18\x02 \x03(\x01\x12\x13\n\x0bneuronYield\x18\x03 \x03(\x01\x12\x12\n\nnumNeurons\x18\x04 \x02(\x03\"\x90\x01\n\x18IndicoDashboardPortStats\x12\x0b\n\x03snr\x18\x01 \x03(\x01\x12\r\n\x05noise\x18\x02 \x03(\x01\x12\x13\n\x0bneuronYield\x18\x03 \x03(\x01\x12\x16\n\x0esortEfficiency\x18\x04 \x02(\x01\x12\x17\n\x0fnumEnabledSites\x18\x05 \x02(\x03\x12\x12\n\nnumNeurons\x18\x06 \x02(\x03\"\xd9\x01\n\x1dIndicoDashboardSiteIndicators\x12\x12\n\nntvChanIdx\x18\x01 \x03(\x05\x12\x0b\n\x03snr\x18\x02 \x03(\x01\x12\r\n\x05noise\x18\x03 \x03(\x01\x12\x13\n\x0bneuronYield\x18\x04 \x03(\x01\x12\x16\n\x0esortEfficiency\x18\x05 \x03(\x01\x12\x11\n\tspikeRate\x18\x06 \x03(\x01\x12\x11\n\tposProbeX\x18\x07 \x03(\x01\x12\x11\n\tposProbeY\x18\x08 \x03(\x01\x12\x11\n\tposProbeZ\x18\t \x03(\x01\x12\x0f\n\x07siteNum\x18\n \x03(\x05\"\xb1\x01\n\x1fIndicoDashboardNeuronIndicators\x12\x10\n\x08neuronID\x18\x01 \x03(\t\x12\x0b\n\x03snr\x18\x02 \x03(\x01\x12\x11\n\tspikeRate\x18\x03 \x03(\x01\x12\x11\n\tposProbeX\x18\x04 \x03(\x01\x12\x11\n\tposProbeY\x18\x05 \x03(\x01\x12\x11\n\tposProbeZ\x18\x06 \x03(\x01\x12\x0f\n\x07siteNum\x18\x07 \x03(\x05\x12\x12\n\nspikeLabel\x18\x08 \x03(\x05\"\xa7\x02\n\x1cSpikeSorterDashboardSiteDesc\x12\x1a\n\x04port\x18\x01 \x02(\x0e\x32\x0c.allego.Port\x12\x12\n\nntvChanIdx\x18\x02 \x02(\x05\x12\x10\n\x08posProbe\x18\x03 \x03(\x01\x12\x10\n\x08posBrain\x18\x04 \x03(\x01\x12\x0e\n\x06snrMax\x18\x05 \x02(\x01\x12\x0e\n\x06snrMin\x18\x06 \x02(\x01\x12\x12\n\nnoiseLevel\x18\x07 \x02(\x01\x12\x12\n\nnumNeurons\x18\x08 \x02(\x03\x12/\n\rindicoCluster\x18\n \x03(\x0b\x32\x18.allego.ClusterMapRecord\x12\x1c\n\x14pileWaveformMinValue\x18\r \x02(\x02\x12\x1c\n\x14pileWaveformMaxValue\x18\x0e \x02(\x02\"\xd2\x01\n\x1eSpikeSorterDashboardNeuronDesc\x12\x1a\n\x04port\x18\x01 \x02(\x0e\x32\x0c.allego.Port\x12&\n\x04\x64\x65sc\x18\x02 \x02(\x0b\x32\x18.allego.NeuronDescriptor\x12\x0b\n\x03snr\x18\x05 \x02(\x01\x12\x0f\n\x07IsihMax\x18\x06 \x02(\x01\x12\x15\n\rIsihArgmaxSec\x18\x07 \x02(\x01\x12\x15\n\rIsihArgmaxIdx\x18\x08 \x02(\x03\x12\x10\n\x08IsihMean\x18\t \x02(\x01\x12\x0e\n\x06IsihSd\x18\n \x02(\x01\"\xfa\x01\n SpikeSorterDashboardNeuronDetail\x12&\n\x04\x64\x65sc\x18\x01 \x02(\x0b\x32\x18.allego.NeuronDescriptor\x12\x18\n\x10templateWaveform\x18\x02 \x03(\x02\x12\x1a\n\x12templateSdWaveform\x18\x03 \x03(\x02\x12\x18\n\x10pileWaveformData\x18\x04 \x03(\x02\x12\x1e\n\x16numSamplesPileWaveform\x18\x05 \x02(\x05\x12\x18\n\x10numPileWaveforms\x18\x06 \x02(\x05\x12$\n\x04isih\x18\x0b \x02(\x0b\x32\x16.allego.NeuronHistData\"\xa9\x01\n\x1eSpikeSorterDashboardSiteDetail\x12\x32\n\x04\x64\x65sc\x18\x01 \x02(\x0b\x32$.allego.SpikeSorterDashboardSiteDesc\x12\x19\n\x11orderedSpikeLabel\x18\x02 \x03(\x05\x12\x38\n\x06neuron\x18\x03 \x03(\x0b\x32(.allego.SpikeSorterDashboardNeuronDetail*\x80\x01\n\x0fSpikeSorterType\x12\x0f\n\x0bSORTER_NULL\x10\x00\x12\x15\n\x11SORTER_VEX_STREAM\x10\x01\x12\x18\n\x14SORTER_SPIKES_STREAM\x10\x02\x12\x13\n\x0fSORTER_VEX_FILE\x10\x03\x12\x16\n\x12SORTER_SPIKES_FILE\x10\x04*\x82\x01\n\x12SpikeSorterCommand\x12\x11\n\rSORTER_CMD_ON\x10\x00\x12\x12\n\x0eSORTER_CMD_OFF\x10\x01\x12\x19\n\x15SORTER_CMD_CLEAR_SORT\x10\x02\x12\x15\n\x11SORTER_CMD_REBASE\x10\x04\x12\x13\n\x0fSORTER_CMD_INIT\x10\x05*/\n\x15SpikeSorterSubCommand\x12\x16\n\x12SORTER_SUBCMD_NULL\x10\x00*@\n\x0fSpikeSorterMode\x12\x16\n\x12SORTER_MODE_STREAM\x10\x00\x12\x15\n\x11SORTER_MODE_BATCH\x10\x01*I\n\x16SpikeSorterStateSystem\x12\n\n\x06SYS_ON\x10\x00\x12\x0b\n\x07SYS_OFF\x10\x01\x12\x16\n\x12SYS_NOT_CONFIGURED\x10\x02*f\n\rSpkSortIOtype\x12\x17\n\x13SPK_SORT_IO_NETWORK\x10\x00\x12\x17\n\x13SPK_SORT_IO_TRAINER\x10\x01\x12#\n\x1fSPK_SORT_IO_NETWORK_AND_TRAINER\x10\x02*>\n\x0fSpkSortIOtarget\x12\x14\n\x10SPK_SORT_IO_FSYS\x10\x00\x12\x15\n\x11SPK_SORT_IO_CLOUD\x10\x01*;\n\rSpkSortIOmode\x12\x14\n\x10SPK_SORT_IO_SAVE\x10\x00\x12\x14\n\x10SPK_SORT_IO_LOAD\x10\x01*\'\n\nRasterMode\x12\x0c\n\x08\x43HANNELS\x10\x00\x12\x0b\n\x07NEURONS\x10\x01*\x89\x01\n\x0b\x44\x61shElement\x12\x11\n\rENABLED_PORTS\x10\x01\x12\x0b\n\x07GENERAL\x10\x02\x12\x0e\n\nPORT_STATS\x10\x03\x12\x0e\n\nSITE_STATS\x10\x04\x12\x0e\n\nSITE_PANEL\x10\x05\x12\x10\n\x0cNEURON_STATS\x10\x06\x12\x10\n\x0cNEURON_PANEL\x10\x07\x12\x06\n\x02\x41I\x10\x08*u\n\x19SpikeSorterCriterionLevel\x12\x10\n\x0cSORTER_LEAST\x10\x00\x12\x0f\n\x0bSORTER_LESS\x10\x01\x12\x13\n\x0fSORTER_BALANCED\x10\x02\x12\x0f\n\x0bSORTER_MORE\x10\x03\x12\x0f\n\x0bSORTER_MOST\x10\x04*\x84\x01\n\x07SS_STAT\x12\x08\n\x04MEAN\x10\x00\x12\x06\n\x02SD\x10\x01\x12\x08\n\x04MODE\x10\x02\x12\x07\n\x03MIN\x10\x03\x12\x07\n\x03MAX\x10\x04\x12\x0e\n\nMODE_COUNT\x10\x05\x12\n\n\x06MEDIAN\x10\x06\x12\x07\n\x03Q25\x10\x07\x12\x07\n\x03Q75\x10\x08\x12\x08\n\x04SKEW\x10\t\x12\x0c\n\x08KURTOSIS\x10\n\x12\x05\n\x01N\x10\x0b*b\n\x10InputFeatureType\x12\x10\n\x0c\x46TR_TYPE_WFM\x10\x00\x12\x10\n\x0c\x46TR_TYPE_SPD\x10\x01\x12\x14\n\x10\x46TR_TYPE_WFM_POS\x10\x02\x12\x14\n\x10\x46TR_TYPE_SPD_POS\x10\x03\x42\x15Z\x13internal/radix/grpc')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'spikesorter_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\023internal/radix/grpc'
  _SPIKESORTERSETDYNAMICREQUEST_VARIANTENTRY._options = None
  _SPIKESORTERSETDYNAMICREQUEST_VARIANTENTRY._serialized_options = b'8\001'
  _SPIKESORTERGETDYNAMICREPLY_VARIANTENTRY._options = None
  _SPIKESORTERGETDYNAMICREPLY_VARIANTENTRY._serialized_options = b'8\001'
  _SPIKESORTERDASHBOARDREPLY_PORTSTATSENTRY._options = None
  _SPIKESORTERDASHBOARDREPLY_PORTSTATSENTRY._serialized_options = b'8\001'
  _SPIKESORTERDASHBOARDREPLY_SITEPANELENTRY._options = None
  _SPIKESORTERDASHBOARDREPLY_SITEPANELENTRY._serialized_options = b'8\001'
  _SPIKESORTERDASHBOARDREPLY_NEURONPANELENTRY._options = None
  _SPIKESORTERDASHBOARDREPLY_NEURONPANELENTRY._serialized_options = b'8\001'
  _SPIKESORTERTYPE._serialized_start=8583
  _SPIKESORTERTYPE._serialized_end=8711
  _SPIKESORTERCOMMAND._serialized_start=8714
  _SPIKESORTERCOMMAND._serialized_end=8844
  _SPIKESORTERSUBCOMMAND._serialized_start=8846
  _SPIKESORTERSUBCOMMAND._serialized_end=8893
  _SPIKESORTERMODE._serialized_start=8895
  _SPIKESORTERMODE._serialized_end=8959
  _SPIKESORTERSTATESYSTEM._serialized_start=8961
  _SPIKESORTERSTATESYSTEM._serialized_end=9034
  _SPKSORTIOTYPE._serialized_start=9036
  _SPKSORTIOTYPE._serialized_end=9138
  _SPKSORTIOTARGET._serialized_start=9140
  _SPKSORTIOTARGET._serialized_end=9202
  _SPKSORTIOMODE._serialized_start=9204
  _SPKSORTIOMODE._serialized_end=9263
  _RASTERMODE._serialized_start=9265
  _RASTERMODE._serialized_end=9304
  _DASHELEMENT._serialized_start=9307
  _DASHELEMENT._serialized_end=9444
  _SPIKESORTERCRITERIONLEVEL._serialized_start=9446
  _SPIKESORTERCRITERIONLEVEL._serialized_end=9563
  _SS_STAT._serialized_start=9566
  _SS_STAT._serialized_end=9698
  _INPUTFEATURETYPE._serialized_start=9700
  _INPUTFEATURETYPE._serialized_end=9798
  _SPIKESORTERLAUNCHREQUEST._serialized_start=64
  _SPIKESORTERLAUNCHREQUEST._serialized_end=362
  _SPIKESORTERISANYACTIVEREQUEST._serialized_start=364
  _SPIKESORTERISANYACTIVEREQUEST._serialized_end=419
  _SPIKESORTERISANYACTIVEREPLY._serialized_start=421
  _SPIKESORTERISANYACTIVEREPLY._serialized_end=500
  _SPIKESORTERCOMMANDREQUEST._serialized_start=503
  _SPIKESORTERCOMMANDREQUEST._serialized_end=773
  _SPIKESORTERSTATE._serialized_start=776
  _SPIKESORTERSTATE._serialized_end=923
  _SPKSORTIOREQ._serialized_start=926
  _SPKSORTIOREQ._serialized_end=1055
  _SPIKESORTERLAUNCHREPLY._serialized_start=1058
  _SPIKESORTERLAUNCHREPLY._serialized_end=1201
  _GETSPIKESORTERIDSREQUEST._serialized_start=1203
  _GETSPIKESORTERIDSREQUEST._serialized_end=1252
  _GETSPIKESORTERIDSREPLY._serialized_start=1254
  _GETSPIKESORTERIDSREPLY._serialized_end=1302
  _SPIKESORTERGETRASTERDATAREQUEST._serialized_start=1305
  _SPIKESORTERGETRASTERDATAREQUEST._serialized_end=1526
  _SPIKESORTERRASTERDATAREPLY._serialized_start=1529
  _SPIKESORTERRASTERDATAREPLY._serialized_end=1686
  _SPIKESORTERSTANDARDREQUEST._serialized_start=1688
  _SPIKESORTERSTANDARDREQUEST._serialized_end=1762
  _SPIKESORTERDASHBOARDREQUEST._serialized_start=1764
  _SPIKESORTERDASHBOARDREQUEST._serialized_end=1859
  _SPIKESORTERDYNAMIC._serialized_start=1862
  _SPIKESORTERDYNAMIC._serialized_end=2252
  _SPIKESORTERDYNAMIC_CRITERIA._serialized_start=2028
  _SPIKESORTERDYNAMIC_CRITERIA._serialized_end=2188
  _SPIKESORTERDYNAMIC_UPDATEPERIODSEC._serialized_start=2190
  _SPIKESORTERDYNAMIC_UPDATEPERIODSEC._serialized_end=2252
  _SPIKESORTERSETDYNAMICREQUEST._serialized_start=2255
  _SPIKESORTERSETDYNAMICREQUEST._serialized_end=2452
  _SPIKESORTERSETDYNAMICREQUEST_VARIANTENTRY._serialized_start=2378
  _SPIKESORTERSETDYNAMICREQUEST_VARIANTENTRY._serialized_end=2452
  _SPIKESORTERGETDYNAMICREPLY._serialized_start=2455
  _SPIKESORTERGETDYNAMICREPLY._serialized_end=2671
  _SPIKESORTERGETDYNAMICREPLY_VARIANTENTRY._serialized_start=2378
  _SPIKESORTERGETDYNAMICREPLY_VARIANTENTRY._serialized_end=2452
  _GETSPIKESORTERPARAMCOMMANDREPLY._serialized_start=2673
  _GETSPIKESORTERPARAMCOMMANDREPLY._serialized_end=2781
  _CLUSTERMAPOUTLIERS._serialized_start=2783
  _CLUSTERMAPOUTLIERS._serialized_end=2834
  _CLUSTERMAPQUARTILES._serialized_start=2836
  _CLUSTERMAPQUARTILES._serialized_end=2893
  _CLUSTERMAPSTATS._serialized_start=2896
  _CLUSTERMAPSTATS._serialized_end=3027
  _CLUSTERMAPRECORD._serialized_start=3029
  _CLUSTERMAPRECORD._serialized_end=3131
  _SPIKESORTSITESTATUS._serialized_start=3134
  _SPIKESORTSITESTATUS._serialized_end=3302
  _SPIKESORTPHASESTATUS._serialized_start=3305
  _SPIKESORTPHASESTATUS._serialized_end=3448
  _SPIKESORTNBRIDXLIST._serialized_start=3450
  _SPIKESORTNBRIDXLIST._serialized_end=3492
  _SPIKESORTERSTATUSREPLY._serialized_start=3495
  _SPIKESORTERSTATUSREPLY._serialized_end=3865
  _SPIKESORTERSTATUSVARIANTSREPLY._serialized_start=3867
  _SPIKESORTERSTATUSVARIANTSREPLY._serialized_end=3949
  _SPIKESORTERCLASSIFIERSPEC._serialized_start=3952
  _SPIKESORTERCLASSIFIERSPEC._serialized_end=4333
  _SPIKESORTERVARIANTSPEC._serialized_start=4335
  _SPIKESORTERVARIANTSPEC._serialized_end=4452
  _SPIKESORTERSPEC._serialized_start=4455
  _SPIKESORTERSPEC._serialized_end=4659
  _SPIKESORTERGETCONFIGREPLY._serialized_start=4662
  _SPIKESORTERGETCONFIGREPLY._serialized_end=4856
  _SPIKESORTERVIZREQUEST._serialized_start=4859
  _SPIKESORTERVIZREQUEST._serialized_end=5131
  _SPIKESORTERVIZREQUEST_VTYPE._serialized_start=5092
  _SPIKESORTERVIZREQUEST_VTYPE._serialized_end=5131
  _SPIKESORTERCLUSTERERPARAMSREQUEST._serialized_start=5134
  _SPIKESORTERCLUSTERERPARAMSREQUEST._serialized_end=5273
  _SPIKESORTERFEATUREPARAMSREQUEST._serialized_start=5276
  _SPIKESORTERFEATUREPARAMSREQUEST._serialized_end=5420
  _SPIKESORTERFEATUREPARAMSREPLY._serialized_start=5423
  _SPIKESORTERFEATUREPARAMSREPLY._serialized_end=5656
  _SPIKESORTERDASHBOARDREPLY._serialized_start=5659
  _SPIKESORTERDASHBOARDREPLY._serialized_end=6353
  _SPIKESORTERDASHBOARDREPLY_PORTSTATSENTRY._serialized_start=6089
  _SPIKESORTERDASHBOARDREPLY_PORTSTATSENTRY._serialized_end=6171
  _SPIKESORTERDASHBOARDREPLY_SITEPANELENTRY._serialized_start=6173
  _SPIKESORTERDASHBOARDREPLY_SITEPANELENTRY._serialized_end=6260
  _SPIKESORTERDASHBOARDREPLY_NEURONPANELENTRY._serialized_start=6262
  _SPIKESORTERDASHBOARDREPLY_NEURONPANELENTRY._serialized_end=6353
  _SPIKESORTERDASHBOARDGENERALREC._serialized_start=6356
  _SPIKESORTERDASHBOARDGENERALREC._serialized_end=6784
  _SPIKESORTERDASHBOARDAI._serialized_start=6787
  _SPIKESORTERDASHBOARDAI._serialized_end=7000
  _INDICODASHBOARDSITESTATS._serialized_start=7002
  _INDICODASHBOARDSITESTATS._serialized_end=7097
  _INDICODASHBOARDPORTSTATS._serialized_start=7100
  _INDICODASHBOARDPORTSTATS._serialized_end=7244
  _INDICODASHBOARDSITEINDICATORS._serialized_start=7247
  _INDICODASHBOARDSITEINDICATORS._serialized_end=7464
  _INDICODASHBOARDNEURONINDICATORS._serialized_start=7467
  _INDICODASHBOARDNEURONINDICATORS._serialized_end=7644
  _SPIKESORTERDASHBOARDSITEDESC._serialized_start=7647
  _SPIKESORTERDASHBOARDSITEDESC._serialized_end=7942
  _SPIKESORTERDASHBOARDNEURONDESC._serialized_start=7945
  _SPIKESORTERDASHBOARDNEURONDESC._serialized_end=8155
  _SPIKESORTERDASHBOARDNEURONDETAIL._serialized_start=8158
  _SPIKESORTERDASHBOARDNEURONDETAIL._serialized_end=8408
  _SPIKESORTERDASHBOARDSITEDETAIL._serialized_start=8411
  _SPIKESORTERDASHBOARDSITEDETAIL._serialized_end=8580
# @@protoc_insertion_point(module_scope)



from pathlib import Path

import pandas as pd
import radiens.utils.config as cfg
from radiens.grpc_radiens import datasource_pb2
from radiens.lib.dataset_metadata import DatasetFileSet
from radiens.utils.constants import TIME_SPEC


class FileSysResponse():
    '''
    Radiens data file set information.
    '''

    def __init__(self, raw_msg: datasource_pb2.CpRmMvLsReply):
        """
        """

        self._num_files = raw_msg.numFiles
        self._num_dsrc = raw_msg.numDsource
        self._num_bytes = raw_msg.numBytes
        self._msg = raw_msg.msg
        self._dsrc: list[DatasetFileSet] = []
        for dsource in raw_msg.dsource:
            self._dsrc.append(DatasetFileSet(dsource))

    @property
    def total_num_files(self) -> int:
        """
        Number of files touched involved with the OS command. It may be greater than or equal to "num_data_sources". 
        """
        return self._num_files

    @property
    def num_data_sources(self) -> int:
        """
        Number of Radiens data sources involved with the OS command. 
        """
        return self._num_dsrc

    @property
    def num_bytes(self) -> int:
        """
        Number of bytes processed across all files by the OS command
        """
        return self._num_bytes

    @property
    def cmd_msg(self) -> str:
        """
        Result message from the OS command
        """
        return self._msg

    @property
    def datasources(self) -> list[DatasetFileSet]:
        """
        List of data sources (aka data file sets) resulting from the OS command.
        """
        return self._dsrc

    @property
    def datasource_table(self) -> pd.DataFrame:
        """
        Table describing channel metadata (index, position, channel type, units, etc)
        """
        d = {'path': [], 'base_name': [], 'type': [], 'dataset_uid': [], 'num_files': [],
             'bytes_primary_data': [], 'bytes_meta_data': [], 'bytes_total': [],
             'timestamp': [], 'num_chan': [], 'dur_sec': [],
             'samp_freq': [], 'checksum': [], 'provenance': []}
        for dsource in self._dsrc:
            d['path'].append(dsource.descriptor.path)
            d['base_name'].append(dsource.descriptor.base_name)
            d['type'].append(dsource.descriptor.file_type)
            d['dataset_uid'].append(dsource.stat.dataset_spec['uid'])
            d['num_files'].append(dsource.stat.num_files)
            d['bytes_primary_data'].append(
                dsource.stat.num_bytes['primary_data'])
            d['bytes_meta_data'].append(dsource.stat.num_bytes['meta_data'])
            d['bytes_total'].append(dsource.stat.num_bytes['total'])
            d['timestamp'].append(dsource.stat.timestamp.strftime(TIME_SPEC))
            d['num_chan'].append(dsource.stat.num_chan)
            d['dur_sec'].append(dsource.stat.dur_sec)
            d['samp_freq'].append(dsource.stat.samp_freq)
            d['checksum'].append(dsource.stat.dataset_spec['checksum'])
            d['provenance'].append(dsource.stat.dataset_spec['provenance'])
        return pd.DataFrame(d)

    def append(self, resp: any):
        """
        appends a FileSysResponse object to the receiver (system reserved)
        """
        if not isinstance(resp, FileSysResponse):
            raise TypeError('resp must be FileSysResponse')
        if len(resp.data_sources) == 0:
            return
        self._num_files = resp.total_num_files
        self._num_dsrc = resp.num_data_sources
        self._num_bytes = resp.num_bytes
        self._msg += ':{}'.format(resp.cmd_msg)
        for dsource in resp.data_sources:
            self._dsrc.append(dsource)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: radiensserver.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from . import datasource_pb2 as datasource__pb2
from . import biointerface_pb2 as biointerface__pb2
from . import spikesorter_pb2 as spikesorter__pb2
from . import radiens_dev_pb2 as radiens__dev__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13radiensserver.proto\x12\x06\x61llego\x1a\x0c\x63ommon.proto\x1a\x10\x64\x61tasource.proto\x1a\x12\x62iointerface.proto\x1a\x11spikesorter.proto\x1a\x11radiens_dev.proto\"K\n\x19RadiensHealthcheckRequest\x12\x1a\n\x12isIncludeWarehouse\x18\x01 \x01(\x08\x12\x12\n\nisDetailed\x18\x02 \x01(\x08\"]\n\x16RadiensHealthcheckSpec\x12\x0e\n\x06\x65rrMsg\x18\x01 \x01(\t\x12\x33\n\toutfitter\x18\x02 \x01(\x0b\x32 .allego.WarehouseHealthcheckSpec\"\x1e\n\x0fRadiensClientId\x12\x0b\n\x03pid\x18\x01 \x02(\x05\"\x91\x01\n\x12SummaSessionsReply\x12?\n\x07session\x18\x01 \x03(\x0b\x32..allego.SummaSessionsReply.SummaCurrentSession\x1a:\n\x13SummaCurrentSession\x12\x0f\n\x07summaID\x18\x01 \x02(\t\x12\x12\n\ndsourceIDs\x18\x02 \x03(\t\"_\n\x14SummaAnalysisRequest\x12\x0f\n\x07summaID\x18\x01 \x01(\t\x12\x12\n\ndsourceIDs\x18\x02 \x03(\t\x12\x11\n\tpktDurSec\x18\x03 \x01(\x01\x12\x0f\n\x07reqPart\x18\x04 \x01(\t\"\xa1\x02\n\x12SummaAnalysisReply\x12\x0f\n\x07summaID\x18\x01 \x01(\t\x12\x15\n\rallDsourceIDs\x18\x02 \x03(\t\x12!\n\x05state\x18\x03 \x01(\x0b\x32\x12.allego.SummaState\x12\x33\n\x0e\x61llDsourceSpec\x18\x04 \x03(\x0b\x32\x1b.allego.SummaDatasourceSpec\x12-\n\x08\x61ggStats\x18\x05 \x01(\x0b\x32\x1b.allego.SummaAggregateStats\x12@\n\x14reqAvailDsourceStats\x18\x06 \x03(\x0b\x32\".allego.SummaDatasourceAnalysisPkg\x12\x1a\n\x12reqAvailDsourceIDs\x18\x07 \x03(\t\"\xd9\x01\n\nSummaState\x12\x0f\n\x07summaID\x18\x01 \x02(\t\x12\x12\n\nisComplete\x18\x02 \x02(\x08\x12\x14\n\x0c\x66racComplete\x18\x03 \x02(\x01\x12\x0b\n\x03msg\x18\x04 \x02(\t\x12\x19\n\x11\x64srcWalltimeStart\x18\x05 \x02(\t\x12\x17\n\x0f\x64srcWalltimeEnd\x18\x06 \x02(\t\x12\x1d\n\x15\x61nalysisWallTimeStart\x18\x07 \x02(\t\x12\x1a\n\x12\x61nalysisElapsedSec\x18\x08 \x02(\x01\x12\x14\n\x0cpacketDurSec\x18\t \x02(\x01\"]\n\x13SummaAggregateStats\x12%\n\x08metricID\x18\x01 \x03(\x0b\x32\x13.allego.KpiMetricID\x12\x1f\n\x04hist\x18\x02 \x03(\x0b\x32\x11.allego.Histogram\"\xbd\x01\n\x1aSummaDatasourceAnalysisPkg\x12%\n\x08metricID\x18\x01 \x03(\x0b\x32\x13.allego.KpiMetricID\x12\x1f\n\x04hist\x18\x02 \x03(\x0b\x32\x11.allego.Histogram\x12!\n\x04recs\x18\x03 \x03(\x0b\x32\x13.allego.DenseMatrix\x12#\n\x08histRecs\x18\x04 \x03(\x0b\x32\x11.allego.Histogram\x12\x0f\n\x07ntvIdxs\x18\x05 \x03(\x03\"~\n\x13SummaDatasourceSpec\x12\x1f\n\x02sG\x18\x01 \x01(\x0b\x32\x13.allego.SignalGroup\x12\x1d\n\x02tR\x18\x02 \x01(\x0b\x32\x11.allego.TimeRange\x12\'\n\x06niSpec\x18\x03 \x01(\x0b\x32\x17.allego.SpikesSpecReply2\xcb\x33\n\x0bRadiensCore\x12R\n\x0bHealthcheck\x12!.allego.RadiensHealthcheckRequest\x1a\x1e.allego.RadiensHealthcheckSpec\"\x00\x12\x41\n\rClientConnect\x12\x17.allego.RadiensClientId\x1a\x15.allego.StandardReply\"\x00\x12\x44\n\x10\x43lientDisconnect\x12\x17.allego.RadiensClientId\x1a\x15.allego.StandardReply\"\x00\x12\x39\n\x05\x43lose\x12\x17.allego.StandardRequest\x1a\x15.allego.StandardReply\"\x00\x12L\n\x10WorkspaceControl\x12\x1f.allego.WorkspaceControlRequest\x1a\x15.allego.StandardReply\"\x00\x12T\n\x11GetRadiensServers\x12 .allego.GetRadiensServersRequest\x1a\x1b.allego.RadiensServersReply\"\x00\x12H\n\x0cGetWorkspace\x12\x1b.allego.GetWorkspaceRequest\x1a\x19.allego.GetWorkspaceReply\"\x00\x12J\n\x13GetRadixEnvironment\x12\x17.allego.StandardRequest\x1a\x18.allego.RadixEnvironment\"\x00\x12R\n\x17GetOfflineLicenseStatus\x12\x17.allego.StandardRequest\x1a\x1c.allego.OfflineLicenseStatus\"\x00\x12H\n\rListDirectory\x12\x1e.allego.ListDataSourcesRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12V\n\x12\x43opyDataSourceFile\x12\'.allego.CopyRemoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12X\n\x14RemoveDataSourceFile\x12\'.allego.CopyRemoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12P\n\x12MoveDataSourceFile\x12!.allego.MoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12N\n\x13SpikesListDirectory\x12\x1e.allego.ListDataSourcesRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12R\n\x0eSpikesCopyFile\x12\'.allego.CopyRemoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12T\n\x10SpikesRemoveFile\x12\'.allego.CopyRemoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12L\n\x0eSpikesMoveFile\x12!.allego.MoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12T\n\x16ManageNodeOrganization\x12\x1c.allego.ManageNodeOrgRequest\x1a\x1a.allego.ManageNodeOrgReply\"\x00\x12K\n\rManageNodeLab\x12\x1c.allego.ManageNodeLabRequest\x1a\x1a.allego.ManageNodeLabReply\"\x00\x12N\n\x0eManageNodeUser\x12\x1d.allego.ManageNodeUserRequest\x1a\x1b.allego.ManageNodeUserReply\"\x00\x12\x45\n\x0fManageNodeProbe\x12\x18.allego.ProbeNodeRequest\x1a\x16.allego.ProbeNodeReply\"\x00\x12\x41\n\x0bManageEdges\x12\x18.allego.WorldEdgeRequest\x1a\x16.allego.WorldEdgeReply\"\x00\x12\x42\n\nQueryWorld\x12\x19.allego.QueryWorldRequest\x1a\x17.allego.QueryWorldReply\"\x00\x12V\n\x1bGetWarehouseSensorComponent\x12\x19.allego.SensorCompRequest\x1a\x1a.allego.GetSensorCompReply\"\x00\x12Q\n\x0fListSensorSpecs\x12\x1e.allego.ListSensorSpecsRequest\x1a\x1c.allego.ListSensorSpecsReply\"\x00\x12>\n\tSetSensor\x12\x18.allego.SetSensorRequest\x1a\x15.allego.StandardReply\"\x00\x12T\n\x14SetSensorPositionTcs\x12#.allego.SetSensorPositionTcsRequest\x1a\x15.allego.StandardReply\"\x00\x12R\n\x13SetSitePositionsTcs\x12\".allego.SetSitePositionsTcsRequest\x1a\x15.allego.StandardReply\"\x00\x12N\n\x11UpdateSignalGroup\x12 .allego.UpdateSignalGroupRequest\x1a\x15.allego.StandardReply\"\x00\x12N\n\x11GetSignalGroupIDs\x12\x17.allego.StandardRequest\x1a\x1e.allego.GetSignalGroupIDsReply\"\x00\x12H\n\x0eGetSorterIDMap\x12\x17.allego.StandardRequest\x1a\x1b.allego.GetSorterIDMapReply\"\x00\x12W\n\x11GetSpikeSorterIDs\x12 .allego.GetSpikeSorterIDsRequest\x1a\x1e.allego.GetSpikeSorterIDsReply\"\x00\x12\x45\n\x0eGetSignalGroup\x12\x1c.allego.SignalGroupIDRequest\x1a\x13.allego.SignalGroup\"\x00\x12O\n\x13GetDataSourceParams\x12\x1c.allego.SignalGroupIDRequest\x1a\x18.allego.DataSourceParams\"\x00\x12=\n\nGetSignals\x12\x19.allego.GetSignalsRequest\x1a\x12.allego.RawSignals\"\x00\x12\x41\n\x0cSetDACStream\x12\x18.allego.DACStreamRequest\x1a\x15.allego.StandardReply\"\x00\x12;\n\tSetDACOff\x12\x15.allego.DACOffRequest\x1a\x15.allego.StandardReply\"\x00\x12\x44\n\x0fGetHDSnapshotPy\x12\x1a.allego.HDSnapshotRequest2\x1a\x13.allego.HDSnapshot2\"\x00\x12@\n\rGetHDSnapshot\x12\x19.allego.HDSnapshotRequest\x1a\x12.allego.HDSnapshot\"\x00\x12+\n\x06GetPSD\x12\x12.allego.PSDRequest\x1a\x0b.allego.PSD\"\x00\x12\x42\n\x0bSetDSPGroup\x12\x1a.allego.SetDSPGroupRequest\x1a\x15.allego.StandardReply\"\x00\x12?\n\x0bGetDSPGroup\x12\x1c.allego.SignalGroupIDRequest\x1a\x10.allego.DSPGroup\"\x00\x12J\n\x0eListDataSource\x12\x19.allego.DataSourceRequest\x1a\x1b.allego.DataSourceStatusMap\"\x00\x12K\n\x11ListDataSourceIDs\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12[\n\x15SetDataSourceFromFile\x12 .allego.DataSourceSetSaveRequest\x1a\x1e.allego.DataSourceSetSaveReply\"\x00\x12I\n\x0f\x43learDataSource\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12J\n\x10RenameDataSource\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12P\n\x1aGetDataSourceKpiFileStatus\x12\x19.allego.DataSourceRequest\x1a\x15.allego.KpiFileStatus\"\x00\x12R\n\x1bGetDataSourceKpiFileStatus2\x12\x19.allego.DataSourceRequest\x1a\x16.allego.KpiFileStatus2\"\x00\x12\x46\n\x0eGetKpiMetadata\x12\x19.allego.DataSourceRequest\x1a\x17.allego.KpiFileMetadata\"\x00\x12?\n\x08KpiClear\x12\x1a.allego.KpiStandardRequest\x1a\x15.allego.StandardReply\"\x00\x12\x43\n\x0cKpiCalculate\x12\x1a.allego.KpiStandardRequest\x1a\x15.allego.StandardReply\"\x00\x12\x45\n\x0cGetKpiStatus\x12\x1b.allego.GetKpiStatusRequest\x1a\x16.allego.KpiStatusReply\"\x00\x12H\n\rKpiGetMetrics\x12\x15.allego.KpiMetricsReq\x1a\x1e.allego.KpiBundlePacketMetrics\"\x00\x12\x46\n\x0fSetKpiPacketDur\x12\x1a.allego.SetKpiParamRequest\x1a\x15.allego.StandardReply\"\x00\x12K\n\x0bSetKpiParam\x12#.allego.SpikeSorterSetParamsRequest\x1a\x15.allego.StandardReply\"\x00\x12S\n\x0bGetKpiParam\x12\x19.allego.DataSourceRequest\x1a\'.allego.GetSpikeSorterParamCommandReply\"\x00\x12W\n\x11SpikesSetFromFile\x12 .allego.DataSourceSetSaveRequest\x1a\x1e.allego.DataSourceSetSaveReply\"\x00\x12\x46\n\x0cSpikesGetIDs\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12H\n\x0eSpikesRenameID\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12S\n\x17SpikesGetAllSummaryInfo\x12\x19.allego.DataSourceRequest\x1a\x1b.allego.DataSourceStatusMap\"\x00\x12\x45\n\rSpikesGetSpec\x12\x19.allego.DataSourceRequest\x1a\x17.allego.SpikesSpecReply\"\x00\x12\x64\n\x13SpikesGetRasterData\x12\'.allego.SpikeSorterGetRasterDataRequest\x1a\".allego.SpikeSorterRasterDataReply\"\x00\x12[\n\x14SpikesGetSpikesDense\x12\x1e.allego.SpikesGetSpikesRequest\x1a!.allego.SpikesSpikeDataDenseReply\"\x00\x12q\n\x15GetSpikeTrainAnalytic\x12+.allego.SpikesGetSpikeTrainAnalyticsRequest\x1a).allego.SpikesGetSpikeTrainAnalyticsReply\"\x00\x12\x42\n\x0eSpikesExportAs\x12\x17.allego.SpikesExportReq\x1a\x15.allego.StandardReply\"\x00\x12I\n\x0fSpikesDeleteIDs\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12\x66\n\x16\x42iointerfaceGetNeurons\x12%.allego.BiointerfaceGetNeuronsRequest\x1a#.allego.BiointerfaceGetNeuronsReply\"\x00\x12l\n\"BiointerfaceSeekEndSpikeTimestamps\x12\x31.allego.BiointerfaceSeekEndSpikeTimestampsRequest\x1a\x11.allego.TimeRange\"\x00\x12Q\n\x0f\x42iointerfaceViz\x12\x1e.allego.BiointerfaceVizRequest\x1a\x1c.allego.BiointerfaceVizReply\"\x00\x12[\n\x1b\x42iointerfaceListSensorSpecs\x12\x1c.allego.SignalGroupIDRequest\x1a\x1c.allego.ListSensorSpecsReply\"\x00\x12J\n\x15\x42iointerfaceSetSensor\x12\x18.allego.SetSensorRequest\x1a\x15.allego.StandardReply\"\x00\x12`\n BiointerfaceSetSensorPositionTcs\x12#.allego.SetSensorPositionTcsRequest\x1a\x15.allego.StandardReply\"\x00\x12^\n\x1f\x42iointerfaceSetSitePositionsTcs\x12\".allego.SetSitePositionsTcsRequest\x1a\x15.allego.StandardReply\"\x00\x12H\n\x0c\x46\x65\x61tureStart\x12\x1f.allego.FeatureStartStopRequest\x1a\x15.allego.StandardReply\"\x00\x12G\n\x0b\x46\x65\x61tureStop\x12\x1f.allego.FeatureStartStopRequest\x1a\x15.allego.StandardReply\"\x00\x12=\n\nGetPrivacy\x12\x17.allego.StandardRequest\x1a\x14.allego.PrivacyReply\"\x00\x12@\n\nSetPrivacy\x12\x19.allego.SetPrivacyRequest\x1a\x15.allego.StandardReply\"\x00\x12J\n\x11\x44\x65leteSignalGroup\x12\x1c.allego.SignalGroupIDRequest\x1a\x15.allego.StandardReply\"\x00\x12K\n\x14SpikesGetSignalGroup\x12\x1c.allego.SignalGroupIDRequest\x1a\x13.allego.SignalGroup\"\x00\x12\x33\n\x0bSetProtocol\x12\x10.allego.Protocol\x1a\x10.allego.Protocol\"\x00\x12:\n\x0bGetProtocol\x12\x17.allego.ProtocolRequest\x1a\x10.allego.Protocol\"\x00\x12\x43\n\x0eRenameProtocol\x12\x1d.allego.RenameProtocolRequest\x1a\x10.allego.Protocol\"\x00\x12J\n\x0fGetAllProtocols\x12\x17.allego.StandardRequest\x1a\x1c.allego.GetAllProtocolsReply\"\x00\x12K\n\rApplyProtocol\x12\x17.allego.ProtocolRequest\x1a\x1d.allego.ApplyProtocolProgress\"\x00\x30\x01\x32\xed\t\n\x13RadiensSpikeSorter1\x12R\n\x0bHealthcheck\x12!.allego.RadiensHealthcheckRequest\x1a\x1e.allego.RadiensHealthcheckSpec\"\x00\x12P\n\x12SpikeSorterCommand\x12!.allego.SpikeSorterCommandRequest\x1a\x15.allego.StandardReply\"\x00\x12R\n\x13SpikeSorterSetParam\x12\".allego.SpikeSorterSetParamRequest\x1a\x15.allego.StandardReply\"\x00\x12T\n\x14SpikeSorterSetParams\x12#.allego.SpikeSorterSetParamsRequest\x1a\x15.allego.StandardReply\"\x00\x12\x64\n\x13SpikeSorterGetParam\x12\".allego.SpikeSorterStandardRequest\x1a\'.allego.GetSpikeSorterParamCommandReply\"\x00\x12U\n\x13SpikeSorterGetState\x12\".allego.SpikeSorterStandardRequest\x1a\x18.allego.SpikeSorterState\"\x00\x12\x63\n\x17SpikeSorterGetDashboard\x12#.allego.SpikeSorterDashboardRequest\x1a!.allego.SpikeSorterDashboardReply\"\x00\x12j\n\x1bSpikeSorterGetFeatureParams\x12\".allego.SpikeSorterStandardRequest\x1a%.allego.SpikeSorterFeatureParamsReply\"\x00\x12I\n\x16SpikeSorterWrangleData\x12\x16.allego.WrangleRequest\x1a\x15.allego.StandardReply\"\x00\x12W\n\x11SpikeSorterLaunch\x12 .allego.SpikeSorterLaunchRequest\x1a\x1e.allego.SpikeSorterLaunchReply\"\x00\x12P\n\x11SpikeSorterDelete\x12\".allego.SpikeSorterStandardRequest\x1a\x15.allego.StandardReply\"\x00\x12N\n\rSpikesGetSpec\x12\".allego.SpikeSorterStandardRequest\x1a\x17.allego.SpikesSpecReply\"\x00\x12\x66\n\x16SpikeSorterIsAnyActive\x12%.allego.SpikeSorterIsAnyActiveRequest\x1a#.allego.SpikeSorterIsAnyActiveReply\"\x00\x12J\n\x17SpikeSorterImportMearec\x12\x14.allego.RecgenImport\x1a\x17.allego.FileDescriptors\"\x00\x32\xe2\x03\n\nDashboards\x12R\n\x0bHealthcheck\x12!.allego.RadiensHealthcheckRequest\x1a\x1e.allego.RadiensHealthcheckSpec\"\x00\x12L\n\x10\x43ommandDashboard\x12\x1f.allego.DashboardCommandRequest\x1a\x15.allego.StandardReply\"\x00\x12S\n\x15LaunchSessionAnalysis\x12\x1c.allego.SummaAnalysisRequest\x1a\x1a.allego.SummaAnalysisReply\"\x00\x12P\n\x12GetSessionAnalysis\x12\x1c.allego.SummaAnalysisRequest\x1a\x1a.allego.SummaAnalysisReply\"\x00\x12\x45\n\x0cListSessions\x12\x17.allego.StandardRequest\x1a\x1a.allego.SummaSessionsReply\"\x00\x12\x44\n\x10\x43learAllSessions\x12\x17.allego.StandardRequest\x1a\x15.allego.StandardReply\"\x00\x32\xef\x05\n\x0bRadiensDev1\x12R\n\x0bHealthcheck\x12!.allego.RadiensHealthcheckRequest\x1a\x1e.allego.RadiensHealthcheckSpec\"\x00\x12S\n\x1aSpkSortTrainerMakeNewModel\x12\x1c.allego.SpkSortTrainerNewReq\x1a\x15.allego.StandardReply\"\x00\x12]\n\x1eSpkSortTrainerLearnHyperParams\x12\".allego.SpkSortLearnHyperParamsReq\x1a\x15.allego.StandardReply\"\x00\x12W\n\x1aSpkSortTrainerProfileModel\x12 .allego.SpkSortTrainerProfileReq\x1a\x15.allego.StandardReply\"\x00\x12W\n#SpkSortTrainerMakeDefaultProdModels\x12\x17.allego.StandardRequest\x1a\x15.allego.StandardReply\"\x00\x12\x64\n$SpkSortTrainerListLocalNetworkModels\x12\x17.allego.StandardRequest\x1a!.allego.SpkSortTrainerStatusReply\"\x00\x12g\n&SpkSortTrainerDeleteLocalNetworkModels\x12$.allego.SpkSortTrainerDeleteModelReq\x1a\x15.allego.StandardReply\"\x00\x12W\n\x17SpkSortTrainerGetStatus\x12\x17.allego.StandardRequest\x1a!.allego.SpkSortTrainerStatusReply\"\x00\x42\x15Z\x13internal/radix/grpc')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'radiensserver_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\023internal/radix/grpc'
  _RADIENSHEALTHCHECKREQUEST._serialized_start=121
  _RADIENSHEALTHCHECKREQUEST._serialized_end=196
  _RADIENSHEALTHCHECKSPEC._serialized_start=198
  _RADIENSHEALTHCHECKSPEC._serialized_end=291
  _RADIENSCLIENTID._serialized_start=293
  _RADIENSCLIENTID._serialized_end=323
  _SUMMASESSIONSREPLY._serialized_start=326
  _SUMMASESSIONSREPLY._serialized_end=471
  _SUMMASESSIONSREPLY_SUMMACURRENTSESSION._serialized_start=413
  _SUMMASESSIONSREPLY_SUMMACURRENTSESSION._serialized_end=471
  _SUMMAANALYSISREQUEST._serialized_start=473
  _SUMMAANALYSISREQUEST._serialized_end=568
  _SUMMAANALYSISREPLY._serialized_start=571
  _SUMMAANALYSISREPLY._serialized_end=860
  _SUMMASTATE._serialized_start=863
  _SUMMASTATE._serialized_end=1080
  _SUMMAAGGREGATESTATS._serialized_start=1082
  _SUMMAAGGREGATESTATS._serialized_end=1175
  _SUMMADATASOURCEANALYSISPKG._serialized_start=1178
  _SUMMADATASOURCEANALYSISPKG._serialized_end=1367
  _SUMMADATASOURCESPEC._serialized_start=1369
  _SUMMADATASOURCESPEC._serialized_end=1495
  _RADIENSCORE._serialized_start=1498
  _RADIENSCORE._serialized_end=8101
  _RADIENSSPIKESORTER1._serialized_start=8104
  _RADIENSSPIKESORTER1._serialized_end=9365
  _DASHBOARDS._serialized_start=9368
  _DASHBOARDS._serialized_end=9850
  _RADIENSDEV1._serialized_start=9853
  _RADIENSDEV1._serialized_end=10604
# @@protoc_insertion_point(module_scope)



from pprint import pprint
import unittest
from pathlib import Path

import radiens.utils.config as cfg


class base_utest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        pass
    
    def test_discover_server_unix(self):
        pass

    

if __name__ == "__main__":
    unittest.main()




import numpy as np
import pandas as pd
from radiens.grpc_radiens import common_pb2
from radiens.utils.constants import PRIMARY_CACHE_STREAM_GROUP_ID, TimeRange
from radiens.utils.enums import (ClientType, MetricID, MetricMode, MetricName,
                                 MetricStats)
from radiens.utils.util import from_dense_matrix, make_time_range


def metric_spec(metric: MetricID) -> dict:
    if metric.name in ['MAX_ABS', 'MAX_ABS_ISI']:
        return {'lbl': '|V_max|', 'units': 'uV'}
    if metric.name in ['MAX_MIN_DIFF_ABS', 'MAX_MIN_DIFF_ABS_ISI']:
        return {'lbl': '|V_pp|', 'units': 'uV'}
    if metric.name in ['SD', 'SD_ISI']:
        return {'lbl': '|V_sd|', 'units': 'uV'}
    if metric.name in ['RMS', 'RMS_ISI']:
        return {'lbl': 'V_rms', 'units': 'uV'}
    if metric.name in ['SNR']:
        return {'lbl': 'SNR', 'units': ''}
    if metric.name in ['EVENT_RATE']:
        return {'lbl': 'Spike Rate', 'units': 'spikes/sec'}
    if metric.name in ['EVENT_MAX_ABS']:
        return {'lbl': 'Spike |V_max|', 'units': 'uV'}
    if metric.name in ['EVENT_MAX_MIN_DIFF_ABS']:
        return {'lbl': 'Spike |V_pp|', 'units': 'uV'}
    if metric.name in ['MEAN_MAX_ABS']:
        return {'lbl': 'Avg. Spike |V_max|', 'units': 'uV'}
    if metric.name in ['EVENT_MEAN_MAX_MIN_DIFF_ABS']:
        return {'lbl': 'Avg. Spike |V_pp|', 'units': 'uV'}
    if metric.name in ['MAX_MIN_DIFF_ABS_AMPLIFIED']:
        return {'lbl': '|V_pp|^2 for V_pp > 2*SD', 'units': 'uV^2'}
    return {'lbl': 'unknown', 'units': 'unknown'}


def sort_metric_ids(metric_ids: list[MetricID]) -> list[MetricID]:
    '''Returns 'metric_ids' list sorted in ascending order by mode and name, respectively. '''
    x = []
    for i in metric_ids:
        x.append([i.mode.value, i.name.name])
    x.sort()
    s = []
    for i in x:
        s.append(MetricID(mode=MetricMode(i[0]), name=MetricName[i[1]]))
    return s


class SignalMetricsStatus():
    '''
    Container for signal metrics status
    '''

    def __init__(self, cache: common_pb2.KpiStatusReply, file: common_pb2.KpiFileStatus2 = None):
        if cache is None and file is None:
            raise ValueError('cache and file cannot both be None')
        self._d = {'file': None, 'cache': None}
        if cache not in [None]:
            self._d['cache'] = {
                'dsource_id': cache.streamGroupId,
                'num_packets_memory': cache.numPacketsMemory, 'num_sigs': cache.numSigs,
                'packet_dur_sec': cache.packetDur, 'update_period_sec': cache.updatePeriod, 'beta': cache.beta, 'is_tracking_cache': cache.isTrackingSignalCache,
                'num_packets': cache.numPackets, 'persistence_sec': cache.persistence, 'memory_bytes': cache.memoryBytes, 'TR': make_time_range(pbTR=cache.tR)}
        if file not in [None]:
            self._d['file'] = {'kpi': {}, 'source': {
            }, 'scanner_mode': 'sprinting' if file.dsource[0].isSprinting else 'off'}
            self._d['file']['kpi']['is_avail'] = file.dsource[0].kpi.isAvail
            self._d['file']['kpi']['is_complete'] = file.dsource[0].kpi.isComplete
            self._d['file']['kpi']['frac_complete'] = file.dsource[0].kpi.fracComplete
            self._d['file']['kpi']['file_path_and_name'] = file.dsource[0].kpi.filePathAndName
            self._d['file']['kpi']['TR'] = make_time_range(
                pbTR=file.dsource[0].kpi.kpiTR)
            self._d['file']['kpi']['packet_dur_sec'] = file.dsource[0].kpi.packetDurSec
            self._d['file']['kpi']['num_sigs'] = file.dsource[0].kpi.numAmp
            self._d['file']['kpi']['bytes_in_bundle_packet'] = file.dsource[0].kpi.bytesInBundlePkt
            self._d['file']['kpi']['num_packets'] = file.dsource[0].kpi.numPackets
            self._d['file']['kpi']['dataset_uid'] = file.dsource[0].kpi.datasetUID
            self._d['file']['source']['file_path_and_name'] = file.dsource[0].kpi.filePathAndName
            self._d['file']['source']['TR'] = make_time_range(
                pbTR=file.dsource[0].source.TR)
            self._d['file']['source']['dataset_uid'] = file.dsource[0].source.datasetUID
            self._d['file']['source']['dsource_id'] = file.dsource[0].source.dsourceID

    @ property
    def time_range(self) -> TimeRange:
        """
        Signal metrics time range of either the backing file (if available) or the backing cache.
        """
        if self._d['file'] not in [None]:
            return self._d['file']['kpi']['TR']
        return self._d['cache']['TR']

    @ property
    def TR(self) -> TimeRange:
        """
        Aliases signal metrics available time range
        """
        return self.time_range

    @ property
    def type(self) -> str:
        """
        Returns 'file' if the signal metrics are backed by a file or 'cache' if backed by a cache.
        """
        return 'file' if self._d['file'] not in [None] else 'cache'

    @ property
    def datasource_id(self) -> str:
        """
        Datasource ID of the linked dataset on the Radiens hub.
        """
        if self._d['file'] not in [None]:
            return self._d['file']['source']['dsource_id']
        return self._d['cache']['dsource_id']

    @ property
    def num_sigs(self) -> int:
        """
        Number of signals.
        """
        if self._d['file'] not in [None]:
            return self._d['file']['kpi']['num_sigs']
        return self._d['cache']['num_sigs']

    @ property
    def packet_dur_sec(self) -> float:
        """
        Packet duration in sec
        """
        if self._d['file'] not in [None]:
            return self._d['file']['kpi']['packet_dur_sec']
        return self._d['cache']['packet_dur_sec']

    @ property
    def num_packets(self) -> int:
        """
        Packet duration in sec
        """
        if self._d['file'] not in [None]:
            return self._d['file']['kpi']['num_packets']
        return self._d['cache']['num_packets']

    @ property
    def is_calculating_metrics(self) -> bool:
        """
        Returns True if radiens hub is calculating the signal metrics
        """
        if self._d['file'] not in [None]:
            return not self._d['file']['kpi']['is_complete']
        return True

    @ property
    def attributes(self) -> dict:
        """
        Signal metrics attributes (dict)
        """
        return self._d

    @ property
    def summary_table(self) -> pd.DataFrame:
        return pd.DataFrame({'dsource_id': [self.datasource_id],
                             'source': [self.type],
                             'time_range': [self.TR.sec],
                             'walltime': [self.TR.walltime],
                             'num_signals': [self.num_sigs],
                             'packet_dur_sec': [self.packet_dur_sec],
                             'num_packetes': [self.num_packets],
                             }).T

    def print(self):
        """
        Prints the status.
        """
        print('Signal metrics status')
        print(self.summary_table)

    def __repr__(self):
        return self.summary_table.__repr__()

    def __str__(self):
        return self.summary_table.__str__()


class SignalMetrics():
    '''
    Container for signal metrics
    '''

    def __init__(self, dsource_id: str,  resp=common_pb2.KpiBundlePacketMetrics):
        self._ntv_idxs = resp.ntvIdxs
        self._d = {'dsource_id': dsource_id,
                   'num_packets': resp.metrics.numPackets, 'num_sigs': resp.metrics.numSigs,
                   'num_metrics': resp.metrics.numMetrics, 'packet_dur_sec': resp.metrics.packetDurSec,
                   'time_range': make_time_range(pbTR=resp.metrics.tR), 'signal_type': common_pb2.SignalType.Name(resp.sigType)}
        self._metric_ids = []
        for m_id in resp.metrics.metric:
            self._metric_ids.append(
                MetricID(mode=MetricMode(m_id.mode), name=MetricName(m_id.name)))
        self._pkt_idxs = resp.metrics.pktIdxs
        self._val = np.reshape(np.ravel(from_dense_matrix(resp.metrics.val)),
                               (self._d['num_packets'], self._d['num_sigs'], self._d['num_metrics']))
        self._stats_packets = MetricStats(mean=resp.metrics.statsPkts.mean, sd=resp.metrics.statsPkts.sD,
                                          min=resp.metrics.statsPkts.min, max=resp.metrics.statsPkts.max)
        self._stats_ = MetricStats(mean=resp.metrics.stats.mean, sd=resp.metrics.stats.sD,
                                   min=resp.metrics.stats.min, max=resp.metrics.stats.max)

    @property
    def _client_type(self) -> ClientType:
        return ClientType.ALLEGO if self._d['dsource_id'] in [
            PRIMARY_CACHE_STREAM_GROUP_ID] else ClientType.VIDERE

    @ property
    def time_range(self) -> TimeRange:
        """
        Signal metrics time range
        """
        return self._d['time_range']

    @ property
    def TR(self) -> TimeRange:
        """
        Aliases signal metrics time range
        """
        return self._d['time_range']

    @ property
    def ntv_idxs(self) -> list:
        """
        Signal metrics native channel indices
        """
        return self._ntv_idxs

    @ property
    def settings(self) -> dict:
        """
        Signal metrics settings (dict)
        """
        return self._d

    @ property
    def metric_ids(self) -> list:
        """
        Ordered list of signal metric IDs.
        """
        return self._metric_ids

    @ property
    def packet_idxs(self) -> list:
        """
        Signal metrics packet indices
        """
        return self._pkt_idxs

    @ property
    def num_packets(self) -> int:
        """
        Signal metrics number of packets
        """
        return self._d['num_packets']

    @ property
    def num_sigs(self) -> int:
        """
        Signal metrics number of signals
        """
        return self._d['num_sigs']

    @ property
    def packet_dur_sec(self) -> int:
        """
        Signal metrics packet duration in seconds
        """
        return self._d['packet_dur_sec']

    @ property
    def val(self) -> np.ndarray:
        """
        Signal metrics settings as multi-dimensional np.ndarray, with dim 0=packet idx, dim 1=channel index, dim 2=metric index.
        The ordered metric IDs are available in obj.metric_ids.

        Examples:

        1. access packet 20, channel 23, and metric 1
            `val[20][23][1]`

        2. access channel 23, metric 1 over all packets:
            `val[:,23,1]`

        3. access metric 1 over all packets and channels:
            `z = y[:,:,1]`  # returns 2-D array, where dim 0=packet index and dim 1=channels
            `np.ravel(z)`  # returns 1-D array with packets flattened

        """
        return self._val

    @property
    def table(self) -> pd.DataFrame:
        return pd.DataFrame({'datasource ID': [self._d['dsource_id']],
                            'signal type': [self._d['signal_type']],
                             'time range (sec)': ['[{:.3f}, {:.3f}]'.format(*self._d['time_range'].sec)],
                             'wall time': ['{}'.format(self._d['time_range'].walltime)],
                             'num signals': [self._d['num_sigs']],
                             'num metrics': [self._d['num_metrics']],
                             'packet dur (sec)': [self._d['packet_dur_sec']],
                             'num packets': [self._d['num_packets']]}).T

    @ property
    def packet_stats(self) -> dict:
        """
        """
        pkt_stats = {}
        return pkt_stats

    def print(self):
        """
        Prints a summary of the metrics.
        """
        print('{}: Signal metrics'.format(self._client_type.name))
        print(self.table)

    def __repr__(self):
        return self.table.__repr__()

    def __str__(self):
        return self.table.__str__()



from pprint import pprint
from tqdm import tqdm
import unittest
import time
from datetime import datetime
from pathlib import Path

from radiens.base_client import BaseClient


class base_utest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls._dir = Path(__file__).parent

    def setUp(self):
        self.client = BaseClient()
        self.radix_data = '~/radix/data'
        self.data_path = '~/radix/data/radiens_test_data'
        self.dataset_0 = '64chan_1s'
        self.tmp_data_path = '~/radix/data/radiens_test_data/tmp'
        self.tmp_dataset_1 = 'tmp_0'

    def tearDown(self) -> None:
        if Path(self.tmp_data_path, self.tmp_dataset_1+'_data.xdat').expanduser().resolve().exists():
            Path(self.tmp_data_path, self.tmp_dataset_1+'_data.xdat').expanduser().resolve().unlink()
            Path(self.tmp_data_path, self.tmp_dataset_1+'_timestamp.xdat').expanduser().resolve().unlink()
            Path(self.tmp_data_path, self.tmp_dataset_1+'.xdat.json').expanduser().resolve().unlink()

    def _has_test_data(self):
        if not Path(self.data_path).expanduser().resolve().exists() or not Path(self.data_path, self.dataset_0+'_data.xdat').expanduser().resolve().exists():
            return False
        return True

    def test_list_dir(self):
        resp = self.client.ls(Path(self.radix_data))
        #self.assertEqual(Path(self.radix_data).expanduser().resolve(), resp.dest_path)

    def test_cp(self):
        if not self._has_test_data():
            self.skipTest('~/radix/data/radiens_test_data is not set up')
        resp = self.client.cp(Path(self.data_path, '64chan_1s.xdat'), Path(self.tmp_data_path, self.tmp_dataset_1))
        #self.assertEqual(Path(self.data_path, 'tmp').expanduser().resolve(), resp.dest_path)

    def test_rm(self):
        if not self._has_test_data():
            self.skipTest('~/radix/data/radiens_test_data is not set up')
        resp = self.client.cp(Path(self.data_path, '64chan_1s.xdat'), Path(self.tmp_data_path, self.tmp_dataset_1))
        resp = self.client.rm(Path(self.tmp_data_path, self.tmp_dataset_1))
        #self.assertEqual(Path(self.tmp_data_path).expanduser().resolve(), resp.dest_path)

    def test_mv(self):
        if not self._has_test_data():
            self.skipTest('~/radix/data/radiens_test_data is not set up')
        resp = self.client.cp(Path(self.data_path, '64chan_1s.xdat'), Path(self.tmp_data_path, 'tmp_to_mov'))
        resp = self.client.mv(Path(self.tmp_data_path, 'tmp_to_mov'), Path(self.tmp_data_path, self.tmp_dataset_1))
        #self.assertEqual(Path(self.data_path, 'tmp').expanduser().resolve(), resp.dest_path)

    def test_set_dataset(self):
        self.client.clear_dataset(dataset_id='all')
        dsource = self.client.link_data_file(Path(self.data_path, '64chan_1s.xdat'))
        self.assertEqual('64chan_1s', dsource.base_name)
        self.assertEqual([16, 17], dsource.time_range.sec.tolist())
        self.assertEqual([320000, 340000], dsource.time_range.timestamp.tolist())
        self.assertEqual(1, len(self.client.get_dataset_ids()))
        self.client.clear_dataset(dataset_id=dsource.id)
        self.assertEqual(0, len(self.client.get_dataset_ids()))


if __name__ == "__main__":
    unittest.main()



"""
radiens CLI
"""
from pathlib import Path

import click
import pandas as pd
from radiens.cli.signal_metrics import metrics
from radiens.cli.signals import signals
from radiens.cli.spike_sorter import sorter
from radiens.utils.enums import ClientType
from radiens.utils.util import time_now


@ click.group(short_help='videre commands')
@ click.pass_context
def videre(ctx):
    '''
    VIDERE command group

    This command group works concurrently with the VIDERE app to control the data acquisition system.
    '''
    ctx.obj['cmd_app'] = ClientType.VIDERE


videre.add_command(sorter)
videre.add_command(metrics)
videre.add_command(signals)


@ videre.command('link', short_help='link a data file to Radiens')
@ click.pass_context
@ click.argument('datafile', type=click.STRING)
@ click.option('-m', '--metrics',  type=click.BOOL, default=True, help='calculate signal metrics', show_default=True)
@ click.option('-f', '--force',  type=click.BOOL, default=False, help='force linking', show_default=True)
@ click.option('-h', '--hub',  type=click.STRING, default='default', help='Radiens hub', show_default=True)
def link_datasource(ctx, datafile, metrics, force, hub):
    '''
    Links the requested data source to the Radiens hub. 
    '''
    try:
        dsource = ctx.obj['videre_client'].link_data_file(
            datafile, calc_metrics=metrics, force=force)
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    click.echo('Linked {} (id={}) to Radiens hub {} at {}'.format(
        Path(datafile), dsource.id, hub, time_now()))


@ videre.command('metadata', short_help='prints meta data on a Radiens data file')
@ click.pass_context
@ click.option('-idx', '--index',  type=click.INT, default=None, help='dataset index from table', show_default=True)
@ click.option('-id', '--dset_id',  type=click.STRING, default=None, help='dataset ID from table', show_default=True)
@ click.option('-p', '--path',  type=click.STRING, default=None, help='path and base name of Radiens file set', show_default=True)
@ click.option('-h', '--hub',  type=click.STRING, default='default', help='Radiens hub', show_default=True)
def dsource_metadata(ctx, index, dset_id, path, hub):
    '''
    Prints table of meta data for one requested linked dataset or Radiens-compatible data file. If --path is specified
    then the requested data file is linked to the Radiens hub.  
    '''
    try:
        resp = ctx.obj['videre_client'].get_data_file_metadata(dataset_idx=index, dataset_id=dset_id,
                                                               path=path, fail_hard=True)
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    click.echo('Linked dataset meta data [hub {}]'.format(hub))
    print(resp.table)


@ videre.command('list', short_help='print table of linked data source on the Radiens hub')
@ click.pass_context
@ click.option('-h', '--hub',  type=click.STRING, default='default', help='Radiens hub', show_default=True)
def list_dsource(ctx, hub):
    '''
    Prints table of currently linked data sources on the Radiens hub
    '''
    try:
        df_table = ctx.obj['videre_client'].get_dataset_ids()
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    click.echo('Linked dataset table [hub {}, N={}]'.format(
        hub, len(df_table)))
    print(df_table)


@ videre.command('unlink', short_help='unlink a data file from the Radiens hub')
@ click.pass_context
@ click.option('-idx', '--index',  type=click.INT, default=None, help='dataset index from table', show_default=True)
@ click.option('-id', '--dset_id',  type=click.STRING, default=None, help='dataset ID from table', show_default=True)
@ click.option('-p', '--path',  type=click.STRING, default=None, help='path and base name of Radiens file set', show_default=True)
@ click.option('-h', '--hub',  type=click.STRING, default='default', help='Radiens hub', show_default=True)
def unlink_dsource(ctx, index, dset_id, path, hub):
    '''
    Unlinks one or more datasets of this Curate session from the Radiens hub.
    Use dataset_id='all' to clear all session datasets.
    '''
    try:
        num_unlinked, df_table = ctx.obj['videre_client'].clear_dataset(dataset_idx=index, dataset_id=dset_id,
                                                                        path=path, fail_hard=True)
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    click.echo('Unlinked {} datasets from hub {}'.format(num_unlinked, hub))
    click.echo('Linked dataset table [hub {}, N={}]'.format(
        hub, len(df_table)))
    print(df_table)


@ videre.command('dashboard', short_help='manage Videre dashboard(s)')
@ click.pass_context
@ click.option('-c', '--close',  type=click.BOOL, default=False, help='close all dashboards', show_default=True)
@ click.option('-h', '--hub',  type=click.STRING, default='default', help='Radiens hub', show_default=True)
def dash_new(ctx, close, hub):
    '''
    Launch a new dashboard or close all dashboards for the Radiens hub. 
    '''
    try:
        ctx.obj['videre_client'].dashboard(close=close, hub_name=hub)
    except Exception as ex:
        click.echo('Error: {}', ex)
        return
    if close:
        click.echo('Closing all dashboards')
    else:
        click.echo('Launching new dashboard...')




from radiens.api import api_allego
from radiens.api.api_utils.util import BaseClient
from radiens.utils.constants import ALLEGO_CORE_ADDR


class WorkspaceClient(BaseClient):
    """
    Workspace client object for AllegoClient
    """

    def __init__(self):
        """ """
        super().__init__()

    @property
    def hubs(self) -> dict:
        """
        dict of active radiens hubs, with hub ID as key.
        """
        return self._hubs()

    @property
    def id(self) -> str:
        """
        UID of this client session.
        """
        return self._id()

    def workspace_save(self, workspace_id=None, tags="", notes=""):
        """
        Saves current workspace

        Parameters:
            workspace_id (str): optional workspace ID
            tags (str): optional tags
            notes (str): optional notes

        Returns:
            None

        Example:
            >>> client.workspace_save(workspace_id='my_wspace', tags='my_tags', notes='my_notes)
            None
        """
        if workspace_id is None:
            return api_allego.workspace_save(ALLEGO_CORE_ADDR, True, tags, notes)
        else:
            return api_allego.workspace_save_as(ALLEGO_CORE_ADDR, workspace_id, True, tags, notes)

    def workspace_switch(self, workspace_id: str):
        """
        Switches to requested workspace

        Parameters:
            workspace_id (str): workspace ID

        Returns:
            None

        Example:
            >>> client.workspace_switch('my_wspace')
            None
        """
        return api_allego.workspace_switch(ALLEGO_CORE_ADDR, workspace_id)

    def workspace_delete(self, workspace_id: str):
        """
        Deletes requested workspace

        Parameters:
            workspace_id (str): workspace ID

        Returns:
            None

        Example:
            >>> client.workspace_delete('my_wspace')
            None
        """
        return api_allego.workspace_delete(ALLEGO_CORE_ADDR, workspace_id)

    def workspace_current(self):
        """
        Returns current workspace ID

        Returns:
            workspaces (pandas.DataFrame)

        Example:
            >>> client.workspace_current()
            df
        """
        return api_allego.workspace_current(ALLEGO_CORE_ADDR)

    def workspace_list(self):
        """
        Returns table of all available workspaces

        Returns:
            workspaces (pandas.DataFrame)

        Example:
            >>> client.workspace_list()
            df
        """
        return api_allego.workspace_list(ALLEGO_CORE_ADDR)



"""
radiens CLI
"""
from pathlib import Path
from pprint import pprint

import click
from radiens.tdt.convert_tdt import convert_tdt


@ click.group(short_help='file system commands')
@ click.option('--hub', default='default', help='radiens hub name')
@ click.pass_context
def filesys(ctx, hub):
    '''
    RADIENS file system commands

    The commands operate on Radiens `data sets`.  A `data set` consists of one or more files that contain
    the primary data and meta data.
    '''
    ctx.obj['hub'] = hub


@ filesys.command(short_help='lists data sets in one or more directories')
@ click.pass_context
@ click.option('-s', '--sort', default='date', help='sort attribute', show_default=True)
@ click.option('-b', '--brief', default=True, type=bool, help='show brief description', show_default=True)
@ click.argument('path', nargs=-1)
def ls(ctx, sort, brief, path):
    '''
    List the Radiens-aware data sets in one or more directories.

    A `data set` represents one set of primary data and its associated meta data in one or more files.
    '''
    path = '.' if len(path) == 0 else path
    try:
        resp = ctx.obj['fsys_client'].ls(
            path, sort_by=sort, hub_name=ctx.obj['hub'])
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    if resp.num_data_sources == 0:
        print('No Radiens data file sets in requested directory')
        return
    df = resp.datasource_table
    df['bytes_total'] = df['bytes_total'] / 1e6
    df['bytes_total'] = df['bytes_total'].map('{:,.3f}'.format)
    if brief:
        pprint(df[['path', 'base_name', 'type', 'bytes_total', 'timestamp']].rename(
            {'base_name': 'name', 'bytes_total': 'size MB'}, axis='columns'))
    else:
        pprint(df[['path', 'base_name', 'type', 'bytes_total', 'timestamp', 'num_chan', 'dur_sec']].rename({'base_name': 'name',
                                                                                                            'bytes_total': 'size MB',
                                                                                                            'num_chan': 'channels',
                                                                                                            'dur_sec': 'dur (sec)'}, axis='columns'))


@ filesys.command(short_help='copy one data file set')
@ click.pass_context
@ click.argument('source', type=click.STRING)
@ click.argument('dest', type=click.STRING)
@ click.option('-f', '--force', default=False, help='force replace of `dest` if it exists', show_default=True)
def cp(ctx, source, dest, force):
    '''
    Copy one Radiens-aware data file set to a 'dest' data file set.
    '''
    try:
        resp = ctx.obj['fsys_client'].cp(Path(source), Path(
            dest), force=force, hub_name=ctx.obj['hub'])
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    if resp.num_data_sources == 0:
        print('No data sets were copied : {resp.cmd_msg}')
        return
    click.echo('Copied {} to {} ({}, {} Mb)'.format(Path(source).expanduser().absolute(), Path(resp.datasources[0].descriptor.path,
                                                                                               resp.datasources[0].descriptor.base_name),
                                                    resp.datasources[0].descriptor.file_type,
                                                    resp.datasources[0].stat.num_bytes['total']))


@ filesys.command(short_help='move one data file set')
@ click.pass_context
@ click.argument('source', type=click.STRING)
@ click.argument('dest', type=click.STRING)
@ click.option('-f', '--force', help='force replace of DEST if it exists', is_flag=True)
def mv(ctx, source, dest, force):
    '''
    Move one Radiens-aware data file set to a 'dest' location.

    source : path to source file set. Use dataset name as in `ls`. Wildcards are allowed.
    dest   : destination location. If `dest` is a directory then the data set name is retained. 

    '''
    try:
        resp = ctx.obj['fsys_client'].mv(Path(source), Path(
            dest), force=force, hub_name=ctx.obj['hub'])
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    if resp.num_data_sources == 0:
        print('No data sets were moved : {resp.cmd_msg}')
        return
    click.echo('Moved {} to {} ({}, {} Mb)'.format(Path(source).expanduser().absolute(), Path(resp.datasources[0].descriptor.path,
                                                                                              resp.datasources[0].descriptor.base_name),
                                                   resp.datasources[0].descriptor.file_type,
                                                   resp.datasources[0].stat.num_bytes['total']))


@ filesys.command(short_help='remove (delete) one data file set')
@ click.pass_context
@ click.argument('source', type=click.STRING)
def rm(ctx, source):
    '''
    Removes (deletes) one Radiens-aware data file set from the file system. This is irreversible. 

    source : path to source file set. Use dataset name as in `ls`. Wildcards are allowed.

    '''
    try:
        resp = ctx.obj['fsys_client'].rm(Path(source), hub_name=ctx.obj['hub'])
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    if resp.num_data_sources == 0:
        print('Requested data set was not found : {resp.cmd_msg}')
        return
    click.echo('Removed {} ({}, {} Mb)'.format(Path(resp.datasources[0].descriptor.path,
                                                    resp.datasources[0].descriptor.base_name),
                                               resp.datasources[0].descriptor.file_type,
                                               resp.datasources[0].stat.num_bytes['total']))


@ filesys.command(short_help='print meta data for one file set')
@ click.pass_context
@ click.argument('source', type=click.STRING)
def metadata(ctx, source):
    '''
    Prints the meta data for one Radiens-aware data file set.

    source : path to source file set. Use dataset name as in `ls`. Wildcards are allowed.

    '''
    try:
        resp = ctx.obj['fsys_client'].get_data_file_metadata(
            Path(source), hub_name=ctx.obj['hub'])
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
    pprint(resp.table)


@ filesys.command(short_help='convert a TDT recording to xdat for use by radiens')
@ click.pass_context
@ click.argument('source', type=click.Path())
@ click.argument('dest', type=click.Path(), nargs=-1)
@ click.option('-f', '--force', help='force replace of DEST if it exists', is_flag=True)
def import_tdt(ctx, source, dest, force):
    '''
    Import a TDT recording source to radiens by converting it to an xdat fileset

    SOURCE : path to source file set. Use dataset name as in `ls`. Wildcards are allowed.\n
    DEST   : destination location

    '''
    try:
        convert_tdt(source, dest, force)
    except Exception as ex:
        click.echo('Error: {}'.format(ex))
        return
